"""
module for testing time budget analysis

pytest -s -vv test_time_budget.py
"""

import pytest
import sys
import json
import os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

print(sys.path)

from boris import time_budget_functions
from boris import db_functions
from boris import config


@pytest.fixture()
def before():
    os.system("rm -rf output")
    os.system("mkdir output")


class Test_time_budget(object):
    """
    start and end times are not taken into account
    """

    def test_time_budget1(self):
        VERIF = [
            {
                "subject": "subject1",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": "0",
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "subject1",
                "behavior": "s",
                "modifiers": "",
                "duration": 16.875,
                "duration_mean": 5.625,
                "duration_stdev": 1.021,
                "number": 3,
                "inter_duration_mean": 2.15,
                "inter_duration_stdev": 0.0,
            },
            {
                "subject": "subject2",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": "0",
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "subject2",
                "behavior": "s",
                "modifiers": "",
                "duration": 7.675,
                "duration_mean": 7.675,
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
        ]

        pj = json.loads(open("files/test.boris").read())

        ethogram = pj[config.ETHOGRAM]
        selected_observations = ["observation #1"]
        parameters = {
            config.SELECTED_SUBJECTS: ["subject1", "subject2"],
            config.SELECTED_BEHAVIORS: ["p", "s"],
            config.INCLUDE_MODIFIERS: False,
            config.EXCLUDE_BEHAVIORS: False,
            "start time": 0,
            "end time": 100.0,
        }

        cursor = db_functions.load_events_in_db(
            pj, parameters[config.SELECTED_SUBJECTS], selected_observations, parameters[config.SELECTED_BEHAVIORS]
        )

        out, categories = time_budget_functions.time_budget_analysis(ethogram, cursor, selected_observations, parameters, by_category=False)

        #  open("/tmp/test_time_budget1.json", "w").write(json.dumps(out))
        assert out == VERIF

    def test_time_budget2(self):
        VERIF = [
            {
                "subject": "subject1",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": "0",
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "subject2",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 2,
                "inter_duration_mean": 0.0,
                "inter_duration_stdev": "NA",
            },
        ]

        pj = json.loads(open("files/test.boris").read())

        ethogram = pj[config.ETHOGRAM]
        selected_observations = ["observation #2"]
        parameters = {
            config.SELECTED_SUBJECTS: ["subject1", "subject2"],
            config.SELECTED_BEHAVIORS: ["p"],
            config.INCLUDE_MODIFIERS: False,
            config.EXCLUDE_BEHAVIORS: False,
            "start time": 0,
            "end time": 100.0,
        }

        cursor = db_functions.load_events_in_db(
            pj, parameters[config.SELECTED_SUBJECTS], selected_observations, parameters[config.SELECTED_BEHAVIORS]
        )

        out, categories = time_budget_functions.time_budget_analysis(ethogram, cursor, selected_observations, parameters, by_category=False)

        # open("/tmp/test_time_budget2.json", "w").write(json.dumps(out))

        assert out == VERIF

    def test_time_budget3(self):
        VERIF = [
            {
                "subject": "No focal subject",
                "behavior": "s",
                "modifiers": "",
                "duration": 59.625,
                "duration_mean": 9.937,
                "duration_stdev": 8.395,
                "number": 6,
                "inter_duration_mean": 49.268,
                "inter_duration_stdev": 84.012,
            },
            {
                "subject": "No focal subject",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 6,
                "inter_duration_mean": 54.083,
                "inter_duration_stdev": 117.79,
            },
        ]

        pj = json.loads(open("files/test.boris").read())

        ethogram = pj[config.ETHOGRAM]
        selected_observations = ["observation #2"]
        parameters = {
            config.SELECTED_SUBJECTS: ["No focal subject"],
            config.SELECTED_BEHAVIORS: ["s", "p"],
            config.INCLUDE_MODIFIERS: True,
            config.EXCLUDE_BEHAVIORS: True,
            "start time": 0,
            "end time": 320.0,
        }

        cursor = db_functions.load_events_in_db(
            pj, parameters[config.SELECTED_SUBJECTS], selected_observations, parameters[config.SELECTED_BEHAVIORS]
        )

        out, categories = time_budget_functions.time_budget_analysis(ethogram, cursor, selected_observations, parameters, by_category=False)

        # open("/tmp/test_time_budget3.json", "w").write(json.dumps(out))

        assert out == VERIF

    def test_time_budget4(self):
        VERIF = [
            {
                "subject": "No focal subject",
                "behavior": "s",
                "modifiers": "",
                "duration": "UNPAIRED",
                "duration_mean": "UNPAIRED",
                "duration_stdev": "UNPAIRED",
                "number": "UNPAIRED",
                "inter_duration_mean": "UNPAIRED",
                "inter_duration_stdev": "UNPAIRED",
            },
            {
                "subject": "No focal subject",
                "behavior": "p",
                "modifiers": "",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 5,
                "inter_duration_mean": 3.272,
                "inter_duration_stdev": 0.309,
            },
        ]

        pj = json.loads(open("files/test.boris").read())

        ethogram = pj[config.ETHOGRAM]
        selected_observations = ["live not paired"]
        parameters = {
            config.SELECTED_SUBJECTS: ["No focal subject"],
            config.SELECTED_BEHAVIORS: ["s", "p"],
            config.INCLUDE_MODIFIERS: True,
            config.EXCLUDE_BEHAVIORS: True,
            "start time": 0,
            "end time": 26.862,
        }

        cursor = db_functions.load_events_in_db(
            pj, parameters[config.SELECTED_SUBJECTS], selected_observations, parameters[config.SELECTED_BEHAVIORS]
        )

        out, categories = time_budget_functions.time_budget_analysis(ethogram, cursor, selected_observations, parameters, by_category=False)

        # open("/tmp/test_time_budget4.json", "w").write(json.dumps(out))

        assert out == VERIF

    def test_time_budget5(self):
        """
        test time budget with modifiers
        """

        VERIF = [
            {
                "subject": "No focal subject",
                "behavior": "q",
                "modifiers": "m1",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "No focal subject",
                "behavior": "q",
                "modifiers": "m2",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "No focal subject",
                "behavior": "q",
                "modifiers": "m3",
                "duration": "NA",
                "duration_mean": "NA",
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "No focal subject",
                "behavior": "r",
                "modifiers": "m1",
                "duration": 8.775,
                "duration_mean": 8.775,
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
            {
                "subject": "No focal subject",
                "behavior": "r",
                "modifiers": "None",
                "duration": 10.4,
                "duration_mean": 10.4,
                "duration_stdev": "NA",
                "number": 1,
                "inter_duration_mean": "NA",
                "inter_duration_stdev": "NA",
            },
        ]

        pj = json.loads(open("files/test.boris").read())

        ethogram = pj[config.ETHOGRAM]
        selected_observations = ["modifiers"]
        parameters = {
            config.SELECTED_SUBJECTS: ["No focal subject"],
            config.SELECTED_BEHAVIORS: ["q", "r"],
            config.INCLUDE_MODIFIERS: True,
            config.EXCLUDE_BEHAVIORS: False,
            "start time": 0,
            "end time": 180,
        }

        cursor = db_functions.load_events_in_db(
            pj,
            parameters[config.SELECTED_SUBJECTS],
            selected_observations,
            parameters[config.SELECTED_BEHAVIORS],
            time_interval=config.TIME_ARBITRARY_INTERVAL,
        )

        out, categories = time_budget_functions.time_budget_analysis(ethogram, cursor, selected_observations, parameters, by_category=False)

        # open("/tmp/test_time_budget5.json", "w").write(json.dumps(out))

        assert out == VERIF



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.
"""

import logging
from . import config as cfg
from PySide6.QtCore import QSize


def update_windows_title(self):
    """
    update the main window title
    """

    if not self.project:
        project_name = ""
    else:
        if self.pj[cfg.PROJECT_NAME]:
            project_name = self.pj[cfg.PROJECT_NAME]
        else:
            if self.projectFileName:
                project_name = f"Unnamed project ({self.projectFileName})"
            else:
                project_name = "Unnamed project"

    self.setWindowTitle(
        f"{self.observationId + ' - ' * (self.observationId != '')}{project_name}{'*' * self.projectChanged}{(' - ' * (project_name != ''))}{cfg.programName}"
    )


def update_menu(self):
    """
    enable/disable menu option
    """
    logging.debug("function: menu_options")

    project_opened = self.project
    observation_is_active = self.observationId != ""
    project_contains_obs = self.pj[cfg.OBSERVATIONS] != {}

    update_windows_title(self)

    self.toolBar.setIconSize(
        QSize(
            self.config_param.get(cfg.TOOLBAR_ICON_SIZE, cfg.DEFAULT_TOOLBAR_ICON_SIZE_VALUE),
            self.config_param.get(cfg.TOOLBAR_ICON_SIZE, cfg.DEFAULT_TOOLBAR_ICON_SIZE_VALUE),
        )
    )

    # enabled if project loaded
    for action in (
        self.actionEdit_project,
        self.actionSave_project,
        self.actionSave_project_as,
        self.actionExport_project,
        self.actionCheck_project,
        self.actionClose_project,
        self.actionNew_observation,
        self.actionRemove_path_from_media_files,
        self.action_create_observations,
        self.action_obs_list,
        self.actionExport_observations_list,
        self.actionExplore_project,
        self.menuExport_events,
        self.actionLoad_observations_file,
        self.actionExportEvents_2,
        self.actionExport_aggregated_events,
        self.menuas_behavioural_sequences,
        self.actionExport_events_as_Praat_TextGrid,
        self.actionJWatcher,
        self.actionCheckStateEvents,
        self.actionCheckStateEventsSingleObs,
        self.actionClose_unpaired_events,
        self.actionRunEventOutside,
    ):
        action.setEnabled(project_opened)

    # observations

    # enabled if project contain one or more observations
    for w in (
        self.actionOpen_observation,
        self.actionEdit_observation_2,
        self.actionView_observation,
        self.actionObservationsList,
        self.action_obs_list,
        self.actionExport_observations_list,
        self.actionCheckStateEvents,
        self.actionExplore_project,
        self.actionClose_unpaired_events,
        self.menuExport_events,
        self.menuCreate_subtitles_2,
        self.actionExtract_events_from_media_files,
        self.actionExtract_frames_from_media_files,
        self.actionRemove_observations,
        self.menuMedia_file_Images_directories,
        # self.actionSet_paths_relative_to_project_directory,
    ):
        w.setEnabled(project_contains_obs)

    # enabled if current observation
    for action in (
        self.actionAdd_event,
        self.actionClose_observation,
        self.actionDelete_all_events,
        self.actionSelect_observations,
        self.actionDelete_selected_events,
        self.actionEdit_event,
        self.actionEdit_event_time,
        self.actionCopy_events,
        self.actionPaste_events,
        self.actionFind_events,
        self.actionFind_replace_events,
        self.actionCloseObs,
        self.actionCurrent_Time_Budget,
        self.actionPlot_current_observation,
        self.actionFind_in_current_obs,
        self.actionEdit_selected_events,
        self.actionPlot_current_time_budget,
    ):
        action.setEnabled(observation_is_active)

    # enabled if media observation
    for action in (
        self.actionMedia_file_information,
        self.actionJumpForward,
        self.actionJumpBackward,
        self.actionJumpTo,
        self.actionZoom_level,
        self.actionRotate_current_video,
        self.actionDisplay_subtitles,
        self.actionPlay,
        self.actionReset,
        self.actionFaster,
        self.actionSlower,
        self.actionNormalSpeed,
        self.actionPrevious,
        self.actionNext,
        self.actionSnapshot,
        self.actionFrame_backward,
        self.actionFrame_forward,
        self.actionVideo_equalizer,
        self.actionShow_spectrogram,
        self.actionShow_the_sound_waveform,
        self.actionPlot_events_in_real_time,
        self.actionShow_data_files,
        self.menuImage_overlay_on_video_2,
        self.actionAdd_image_overlay_on_video,
        self.actionRemove_image_overlay,
        self.actionAdd_frame_indexes,
    ):
        action.setEnabled(self.playerType == cfg.MEDIA)

    # geometric measurements
    self.action_geometric_measurements.setEnabled(observation_is_active and self.geometric_measurements_mode is False)
    self.actionCoding_pad.setEnabled(observation_is_active)
    self.actionSubjects_pad.setEnabled(observation_is_active)
    self.actionBehaviors_coding_map.setEnabled(observation_is_active)

    for action in (
        self.actionJumpForward,
        self.actionJumpBackward,
        self.actionJumpTo,
        self.actionReset,
        self.actionPrevious,
        self.actionNext,
        self.actionSnapshot,
        self.actionFrame_backward,
        self.actionFrame_forward,
    ):
        action.setEnabled(self.playerType in [cfg.MEDIA, cfg.IMAGES])

    # Analysis
    for w in [
        self.actionTime_budget,
        self.actionTime_budget_by_behaviors_category,
        self.actionTime_budget_report,
        self.action_behavior_binary_table,
        self.action_advanced_event_filtering,
        self.action_latency,
        self.action_cooccurence,
        self.menuPlot_events,
        self.menuInter_rater_reliability,
        self.menuSimilarities,
        self.menuCreate_transitions_matrix,
        self.actionSynthetic_binned_time_budget,
    ]:
        w.setEnabled(project_contains_obs)

    # statusbar labels
    for w in (self.lbTimeOffset, self.lb_obs_time_interval):
        w.setVisible(self.playerType == cfg.MEDIA)

    logging.debug("function: menu_options finished")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'core.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QDockWidget, QHBoxLayout,
    QHeaderView, QLabel, QMainWindow, QMenu,
    QMenuBar, QPushButton, QSizePolicy, QSpacerItem,
    QSplitter, QStatusBar, QTableView, QTableWidget,
    QTableWidgetItem, QToolBar, QVBoxLayout, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1509, 813)
        MainWindow.setDockOptions(QMainWindow.DockOption.AllowNestedDocks|QMainWindow.DockOption.AllowTabbedDocks|QMainWindow.DockOption.AnimatedDocks)
        MainWindow.setUnifiedTitleAndToolBarOnMac(False)
        self.actionDocumentation = QAction(MainWindow)
        self.actionDocumentation.setObjectName(u"actionDocumentation")
        self.actionAbout = QAction(MainWindow)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionQuit = QAction(MainWindow)
        self.actionQuit.setObjectName(u"actionQuit")
        self.actionPause = QAction(MainWindow)
        self.actionPause.setObjectName(u"actionPause")
        self.actionPlay = QAction(MainWindow)
        self.actionPlay.setObjectName(u"actionPlay")
        self.actionOpen_video_file = QAction(MainWindow)
        self.actionOpen_video_file.setObjectName(u"actionOpen_video_file")
        self.actionReset = QAction(MainWindow)
        self.actionReset.setObjectName(u"actionReset")
        self.actionFaster = QAction(MainWindow)
        self.actionFaster.setObjectName(u"actionFaster")
        self.actionFaster.setEnabled(True)
        self.actionSlower = QAction(MainWindow)
        self.actionSlower.setObjectName(u"actionSlower")
        self.actionSlower.setEnabled(True)
        self.actionJumpForward = QAction(MainWindow)
        self.actionJumpForward.setObjectName(u"actionJumpForward")
        self.actionLoad_configuration = QAction(MainWindow)
        self.actionLoad_configuration.setObjectName(u"actionLoad_configuration")
        self.actionDelete_selected_events = QAction(MainWindow)
        self.actionDelete_selected_events.setObjectName(u"actionDelete_selected_events")
        self.actionDelete_all_events = QAction(MainWindow)
        self.actionDelete_all_events.setObjectName(u"actionDelete_all_events")
        self.actionSort_observations = QAction(MainWindow)
        self.actionSort_observations.setObjectName(u"actionSort_observations")
        self.actionLoad_observations_file = QAction(MainWindow)
        self.actionLoad_observations_file.setObjectName(u"actionLoad_observations_file")
        self.actionSelect_observations = QAction(MainWindow)
        self.actionSelect_observations.setObjectName(u"actionSelect_observations")
        self.actionConfigure_states_and_events = QAction(MainWindow)
        self.actionConfigure_states_and_events.setObjectName(u"actionConfigure_states_and_events")
        self.actionConfigure_states_and_events.setEnabled(False)
        self.actionEdit_event = QAction(MainWindow)
        self.actionEdit_event.setObjectName(u"actionEdit_event")
        self.actionLoad_configuration_file = QAction(MainWindow)
        self.actionLoad_configuration_file.setObjectName(u"actionLoad_configuration_file")
        self.actionMedia_file_information = QAction(MainWindow)
        self.actionMedia_file_information.setObjectName(u"actionMedia_file_information")
        self.actionStart_live_observation = QAction(MainWindow)
        self.actionStart_live_observation.setObjectName(u"actionStart_live_observation")
        self.actionNew_project = QAction(MainWindow)
        self.actionNew_project.setObjectName(u"actionNew_project")
        self.actionTime_budget = QAction(MainWindow)
        self.actionTime_budget.setObjectName(u"actionTime_budget")
        self.actionTime_budget.setEnabled(True)
        self.actionTime_budget.setIconVisibleInMenu(True)
        self.actionSave_project = QAction(MainWindow)
        self.actionSave_project.setObjectName(u"actionSave_project")
        self.actionOpen_project = QAction(MainWindow)
        self.actionOpen_project.setObjectName(u"actionOpen_project")
        self.actionSet_offset = QAction(MainWindow)
        self.actionSet_offset.setObjectName(u"actionSet_offset")
        self.actionEdit_project = QAction(MainWindow)
        self.actionEdit_project.setObjectName(u"actionEdit_project")
        self.actionSave_project_as = QAction(MainWindow)
        self.actionSave_project_as.setObjectName(u"actionSave_project_as")
        self.actionPreferences = QAction(MainWindow)
        self.actionPreferences.setObjectName(u"actionPreferences")
        self.actionNew_observation = QAction(MainWindow)
        self.actionNew_observation.setObjectName(u"actionNew_observation")
        self.actionSave_observation = QAction(MainWindow)
        self.actionSave_observation.setObjectName(u"actionSave_observation")
        self.actionClose_observation = QAction(MainWindow)
        self.actionClose_observation.setObjectName(u"actionClose_observation")
        self.actionEdit_current_observation = QAction(MainWindow)
        self.actionEdit_current_observation.setObjectName(u"actionEdit_current_observation")
        self.actionEdit_current_observation.setEnabled(False)
        self.actionOpen_observation_2 = QAction(MainWindow)
        self.actionOpen_observation_2.setObjectName(u"actionOpen_observation_2")
        self.actionOpen_observation_2.setEnabled(False)
        self.actionOpen_observation_2.setVisible(False)
        self.actionAdd_event = QAction(MainWindow)
        self.actionAdd_event.setObjectName(u"actionAdd_event")
        self.actionDeselectCurrentSubject = QAction(MainWindow)
        self.actionDeselectCurrentSubject.setObjectName(u"actionDeselectCurrentSubject")
        self.actionNext = QAction(MainWindow)
        self.actionNext.setObjectName(u"actionNext")
        self.actionNext.setIconVisibleInMenu(False)
        self.actionPrevious = QAction(MainWindow)
        self.actionPrevious.setObjectName(u"actionPrevious")
        self.actionJumpTo = QAction(MainWindow)
        self.actionJumpTo.setObjectName(u"actionJumpTo")
        self.actionJumpTo.setEnabled(True)
        self.actionJumpBackward = QAction(MainWindow)
        self.actionJumpBackward.setObjectName(u"actionJumpBackward")
        self.actionEdit_observation = QAction(MainWindow)
        self.actionEdit_observation.setObjectName(u"actionEdit_observation")
        self.actionEdit_observation.setEnabled(False)
        self.actionEdit_observation.setVisible(False)
        self.actionCheckUpdate = QAction(MainWindow)
        self.actionCheckUpdate.setObjectName(u"actionCheckUpdate")
        self.actionClose_project = QAction(MainWindow)
        self.actionClose_project.setObjectName(u"actionClose_project")
        self.actionObservationsList = QAction(MainWindow)
        self.actionObservationsList.setObjectName(u"actionObservationsList")
        self.actionNormalSpeed = QAction(MainWindow)
        self.actionNormalSpeed.setObjectName(u"actionNormalSpeed")
        self.actionSnapshot = QAction(MainWindow)
        self.actionSnapshot.setObjectName(u"actionSnapshot")
        self.actionFrame_by_frame = QAction(MainWindow)
        self.actionFrame_by_frame.setObjectName(u"actionFrame_by_frame")
        self.actionFrame_by_frame.setCheckable(True)
        self.actionFrame_by_frame.setVisible(False)
        self.actionAggregatedEventsTabularFormat = QAction(MainWindow)
        self.actionAggregatedEventsTabularFormat.setObjectName(u"actionAggregatedEventsTabularFormat")
        self.actionOpen_observation = QAction(MainWindow)
        self.actionOpen_observation.setObjectName(u"actionOpen_observation")
        self.actionAaaa = QAction(MainWindow)
        self.actionAaaa.setObjectName(u"actionAaaa")
        self.menuCreate_subtitles_2 = QAction(MainWindow)
        self.menuCreate_subtitles_2.setObjectName(u"menuCreate_subtitles_2")
        self.actionUser_guide = QAction(MainWindow)
        self.actionUser_guide.setObjectName(u"actionUser_guide")
        self.actionEdit_observation_2 = QAction(MainWindow)
        self.actionEdit_observation_2.setObjectName(u"actionEdit_observation_2")
        self.actionCheckStateEvents = QAction(MainWindow)
        self.actionCheckStateEvents.setObjectName(u"actionCheckStateEvents")
        self.actionRunEventOutside = QAction(MainWindow)
        self.actionRunEventOutside.setObjectName(u"actionRunEventOutside")
        self.actionEdit_selected_events = QAction(MainWindow)
        self.actionEdit_selected_events.setObjectName(u"actionEdit_selected_events")
        self.actionShow_spectrogram = QAction(MainWindow)
        self.actionShow_spectrogram.setObjectName(u"actionShow_spectrogram")
        self.actionExport_events_as_Praat_TextGrid = QAction(MainWindow)
        self.actionExport_events_as_Praat_TextGrid.setObjectName(u"actionExport_events_as_Praat_TextGrid")
        self.actionExtract_events_from_media_files = QAction(MainWindow)
        self.actionExtract_events_from_media_files.setObjectName(u"actionExtract_events_from_media_files")
        self.action_geometric_measurements = QAction(MainWindow)
        self.action_geometric_measurements.setObjectName(u"action_geometric_measurements")
        self.actionFrame_forward = QAction(MainWindow)
        self.actionFrame_forward.setObjectName(u"actionFrame_forward")
        self.actionFrame_backward = QAction(MainWindow)
        self.actionFrame_backward.setObjectName(u"actionFrame_backward")
        self.actionFilterBehaviors = QAction(MainWindow)
        self.actionFilterBehaviors.setObjectName(u"actionFilterBehaviors")
        self.actionShowAllBehaviors = QAction(MainWindow)
        self.actionShowAllBehaviors.setObjectName(u"actionShowAllBehaviors")
        self.actionExport_aggregated_events = QAction(MainWindow)
        self.actionExport_aggregated_events.setObjectName(u"actionExport_aggregated_events")
        self.actionCoding_pad = QAction(MainWindow)
        self.actionCoding_pad.setObjectName(u"actionCoding_pad")
        self.actionTime_budget_by_behaviors_category = QAction(MainWindow)
        self.actionTime_budget_by_behaviors_category.setObjectName(u"actionTime_budget_by_behaviors_category")
        self.actionExport_events_as_SDIS_file = QAction(MainWindow)
        self.actionExport_events_as_SDIS_file.setObjectName(u"actionExport_events_as_SDIS_file")
        self.actionRecode_resize_video = QAction(MainWindow)
        self.actionRecode_resize_video.setObjectName(u"actionRecode_resize_video")
        self.actionMedia_file_information_2 = QAction(MainWindow)
        self.actionMedia_file_information_2.setObjectName(u"actionMedia_file_information_2")
        self.actionCreate_transitions_flow_diagram = QAction(MainWindow)
        self.actionCreate_transitions_flow_diagram.setObjectName(u"actionCreate_transitions_flow_diagram")
        self.actionCreate_transitions_flow_diagram_2 = QAction(MainWindow)
        self.actionCreate_transitions_flow_diagram_2.setObjectName(u"actionCreate_transitions_flow_diagram_2")
        self.actionAll_transitions = QAction(MainWindow)
        self.actionAll_transitions.setObjectName(u"actionAll_transitions")
        self.actionNumber_of_transitions = QAction(MainWindow)
        self.actionNumber_of_transitions.setObjectName(u"actionNumber_of_transitions")
        self.actionFrequencies_of_transitions_after_behaviors = QAction(MainWindow)
        self.actionFrequencies_of_transitions_after_behaviors.setObjectName(u"actionFrequencies_of_transitions_after_behaviors")
        self.actionFind_replace_events = QAction(MainWindow)
        self.actionFind_replace_events.setObjectName(u"actionFind_replace_events")
        self.actionFind_events = QAction(MainWindow)
        self.actionFind_events.setObjectName(u"actionFind_events")
        self.actionHow_to_cite_BORIS = QAction(MainWindow)
        self.actionHow_to_cite_BORIS.setObjectName(u"actionHow_to_cite_BORIS")
        self.actionZoom1_1_1 = QAction(MainWindow)
        self.actionZoom1_1_1.setObjectName(u"actionZoom1_1_1")
        self.actionZoom1_1_1.setCheckable(True)
        self.actionZoom1_2_1 = QAction(MainWindow)
        self.actionZoom1_2_1.setObjectName(u"actionZoom1_2_1")
        self.actionZoom1_2_1.setCheckable(True)
        self.actionZoom1_1_2 = QAction(MainWindow)
        self.actionZoom1_1_2.setObjectName(u"actionZoom1_1_2")
        self.actionZoom1_1_2.setCheckable(True)
        self.actionZoom1_1_4 = QAction(MainWindow)
        self.actionZoom1_1_4.setObjectName(u"actionZoom1_1_4")
        self.actionZoom1_1_4.setCheckable(True)
        self.actionZoom1_fitwindow = QAction(MainWindow)
        self.actionZoom1_fitwindow.setObjectName(u"actionZoom1_fitwindow")
        self.actionZoom1_fitwindow.setCheckable(True)
        self.actionZoom2_fitwindow = QAction(MainWindow)
        self.actionZoom2_fitwindow.setObjectName(u"actionZoom2_fitwindow")
        self.actionZoom2_fitwindow.setCheckable(True)
        self.actionZoom2_1_4 = QAction(MainWindow)
        self.actionZoom2_1_4.setObjectName(u"actionZoom2_1_4")
        self.actionZoom2_1_4.setCheckable(True)
        self.actionZoom2_1_2 = QAction(MainWindow)
        self.actionZoom2_1_2.setObjectName(u"actionZoom2_1_2")
        self.actionZoom2_1_2.setCheckable(True)
        self.actionZoom2_1_1 = QAction(MainWindow)
        self.actionZoom2_1_1.setObjectName(u"actionZoom2_1_1")
        self.actionZoom2_1_1.setCheckable(True)
        self.actionZoom2_2_1 = QAction(MainWindow)
        self.actionZoom2_2_1.setObjectName(u"actionZoom2_2_1")
        self.actionZoom2_2_1.setCheckable(True)
        self.actionFilterSubjects = QAction(MainWindow)
        self.actionFilterSubjects.setObjectName(u"actionFilterSubjects")
        self.actionShowAllSubjects = QAction(MainWindow)
        self.actionShowAllSubjects.setObjectName(u"actionShowAllSubjects")
        self.actionSend_project = QAction(MainWindow)
        self.actionSend_project.setObjectName(u"actionSend_project")
        self.actionViewBehavior = QAction(MainWindow)
        self.actionViewBehavior.setObjectName(u"actionViewBehavior")
        self.actionBehaviors_coding_map = QAction(MainWindow)
        self.actionBehaviors_coding_map.setObjectName(u"actionBehaviors_coding_map")
        self.action_create_behaviors_coding_map = QAction(MainWindow)
        self.action_create_behaviors_coding_map.setObjectName(u"action_create_behaviors_coding_map")
        self.action_create_modifiers_coding_map = QAction(MainWindow)
        self.action_create_modifiers_coding_map.setObjectName(u"action_create_modifiers_coding_map")
        self.actionPlot_events1 = QAction(MainWindow)
        self.actionPlot_events1.setObjectName(u"actionPlot_events1")
        self.actionPlot_events2 = QAction(MainWindow)
        self.actionPlot_events2.setObjectName(u"actionPlot_events2")
        self.actionCohen_s_kappa = QAction(MainWindow)
        self.actionCohen_s_kappa.setObjectName(u"actionCohen_s_kappa")
        self.actionTime_budget_report = QAction(MainWindow)
        self.actionTime_budget_report.setObjectName(u"actionTime_budget_report")
        self.actionCheckStateEventsSingleObs = QAction(MainWindow)
        self.actionCheckStateEventsSingleObs.setObjectName(u"actionCheckStateEventsSingleObs")
        self.actionSubjects_pad = QAction(MainWindow)
        self.actionSubjects_pad.setObjectName(u"actionSubjects_pad")
        self.actionShow_data_files = QAction(MainWindow)
        self.actionShow_data_files.setObjectName(u"actionShow_data_files")
        self.actionConverters_manager = QAction(MainWindow)
        self.actionConverters_manager.setObjectName(u"actionConverters_manager")
        self.actionView_observation = QAction(MainWindow)
        self.actionView_observation.setObjectName(u"actionView_observation")
        self.actionRemove_path_from_media_files = QAction(MainWindow)
        self.actionRemove_path_from_media_files.setObjectName(u"actionRemove_path_from_media_files")
        self.actionRemove_path_from_media_files.setVisible(False)
        self.actionCheck_project = QAction(MainWindow)
        self.actionCheck_project.setObjectName(u"actionCheck_project")
        self.actionBehavior_bar_plot = QAction(MainWindow)
        self.actionBehavior_bar_plot.setObjectName(u"actionBehavior_bar_plot")
        self.action_obs_list = QAction(MainWindow)
        self.action_obs_list.setObjectName(u"action_obs_list")
        self.actionTest = QAction(MainWindow)
        self.actionTest.setObjectName(u"actionTest")
        self.actionTest_stb2 = QAction(MainWindow)
        self.actionTest_stb2.setObjectName(u"actionTest_stb2")
        self.actionClose_unpaired_events = QAction(MainWindow)
        self.actionClose_unpaired_events.setObjectName(u"actionClose_unpaired_events")
        self.actionRotate_video = QAction(MainWindow)
        self.actionRotate_video.setObjectName(u"actionRotate_video")
        self.actionExport_observations_list = QAction(MainWindow)
        self.actionExport_observations_list.setObjectName(u"actionExport_observations_list")
        self.actionExportEvents_2 = QAction(MainWindow)
        self.actionExportEvents_2.setObjectName(u"actionExportEvents_2")
        self.actionJWatcher = QAction(MainWindow)
        self.actionJWatcher.setObjectName(u"actionJWatcher")
        self.actionCloseObs = QAction(MainWindow)
        self.actionCloseObs.setObjectName(u"actionCloseObs")
        self.actionCloseObs.setEnabled(True)
        self.actionFilter_events = QAction(MainWindow)
        self.actionFilter_events.setObjectName(u"actionFilter_events")
        self.actionShow_all_events = QAction(MainWindow)
        self.actionShow_all_events.setObjectName(u"actionShow_all_events")
        self.actionEdit_event_time = QAction(MainWindow)
        self.actionEdit_event_time.setObjectName(u"actionEdit_event_time")
        self.actionCopy_events = QAction(MainWindow)
        self.actionCopy_events.setObjectName(u"actionCopy_events")
        self.actionPaste_events = QAction(MainWindow)
        self.actionPaste_events.setObjectName(u"actionPaste_events")
        self.actionNoldus_Observer_template = QAction(MainWindow)
        self.actionNoldus_Observer_template.setObjectName(u"actionNoldus_Observer_template")
        self.actionExplore_project = QAction(MainWindow)
        self.actionExplore_project.setObjectName(u"actionExplore_project")
        self.actionShow_the_sound_waveform = QAction(MainWindow)
        self.actionShow_the_sound_waveform.setObjectName(u"actionShow_the_sound_waveform")
        self.actionNeedleman_Wunsch = QAction(MainWindow)
        self.actionNeedleman_Wunsch.setObjectName(u"actionNeedleman_Wunsch")
        self.actionExtract_frames_from_media_files = QAction(MainWindow)
        self.actionExtract_frames_from_media_files.setObjectName(u"actionExtract_frames_from_media_files")
        self.action_behavior_binary_table = QAction(MainWindow)
        self.action_behavior_binary_table.setObjectName(u"action_behavior_binary_table")
        self.action_behavior_binary_table.setEnabled(True)
        self.actionPlot_current_observation = QAction(MainWindow)
        self.actionPlot_current_observation.setObjectName(u"actionPlot_current_observation")
        self.actionFind_in_current_obs = QAction(MainWindow)
        self.actionFind_in_current_obs.setObjectName(u"actionFind_in_current_obs")
        self.action_advanced_event_filtering = QAction(MainWindow)
        self.action_advanced_event_filtering.setObjectName(u"action_advanced_event_filtering")
        self.action_block_dockwidgets = QAction(MainWindow)
        self.action_block_dockwidgets.setObjectName(u"action_block_dockwidgets")
        self.action_block_dockwidgets.setCheckable(True)
        self.actionseparated_subjects = QAction(MainWindow)
        self.actionseparated_subjects.setObjectName(u"actionseparated_subjects")
        self.actiongrouped_subjects = QAction(MainWindow)
        self.actiongrouped_subjects.setObjectName(u"actiongrouped_subjects")
        self.actionCurrent_Time_Budget = QAction(MainWindow)
        self.actionCurrent_Time_Budget.setObjectName(u"actionCurrent_Time_Budget")
        self.actionSynthetic_binned_time_budget = QAction(MainWindow)
        self.actionSynthetic_binned_time_budget.setObjectName(u"actionSynthetic_binned_time_budget")
        self.actionZoom_level = QAction(MainWindow)
        self.actionZoom_level.setObjectName(u"actionZoom_level")
        self.actionDisplay_subtitles = QAction(MainWindow)
        self.actionDisplay_subtitles.setObjectName(u"actionDisplay_subtitles")
        self.actionAdd_image_overlay_on_video = QAction(MainWindow)
        self.actionAdd_image_overlay_on_video.setObjectName(u"actionAdd_image_overlay_on_video")
        self.actionRemove_image_overlay = QAction(MainWindow)
        self.actionRemove_image_overlay.setObjectName(u"actionRemove_image_overlay")
        self.actionPlot_events_in_real_time = QAction(MainWindow)
        self.actionPlot_events_in_real_time.setObjectName(u"actionPlot_events_in_real_time")
        self.actionVideo_equalizer = QAction(MainWindow)
        self.actionVideo_equalizer.setObjectName(u"actionVideo_equalizer")
        self.actionRemove_observations = QAction(MainWindow)
        self.actionRemove_observations.setObjectName(u"actionRemove_observations")
        self.action_latency = QAction(MainWindow)
        self.action_latency.setObjectName(u"action_latency")
        self.actionUndo = QAction(MainWindow)
        self.actionUndo.setObjectName(u"actionUndo")
        self.actionaaa = QAction(MainWindow)
        self.actionaaa.setObjectName(u"actionaaa")
        self.action_media_file_and_images_directories_relative_path = QAction(MainWindow)
        self.action_media_file_and_images_directories_relative_path.setObjectName(u"action_media_file_and_images_directories_relative_path")
        self.action_data_files_relative_path = QAction(MainWindow)
        self.action_data_files_relative_path.setObjectName(u"action_data_files_relative_path")
        self.action_remove_media_files_and_images_directories_path = QAction(MainWindow)
        self.action_remove_media_files_and_images_directories_path.setObjectName(u"action_remove_media_files_and_images_directories_path")
        self.action_remove_data_files_path = QAction(MainWindow)
        self.action_remove_data_files_path.setObjectName(u"action_remove_data_files_path")
        self.actionPlot_current_time_budget = QAction(MainWindow)
        self.actionPlot_current_time_budget.setObjectName(u"actionPlot_current_time_budget")
        self.actionMerge_media_files = QAction(MainWindow)
        self.actionMerge_media_files.setObjectName(u"actionMerge_media_files")
        self.actionExport_project = QAction(MainWindow)
        self.actionExport_project.setObjectName(u"actionExport_project")
        self.actionConfigure_twEvents_columns = QAction(MainWindow)
        self.actionConfigure_twEvents_columns.setObjectName(u"actionConfigure_twEvents_columns")
        self.actionRotate_current_video = QAction(MainWindow)
        self.actionRotate_current_video.setObjectName(u"actionRotate_current_video")
        self.action_deinterlace = QAction(MainWindow)
        self.action_deinterlace.setObjectName(u"action_deinterlace")
        self.action_deinterlace.setCheckable(True)
        self.action_cooccurence = QAction(MainWindow)
        self.action_cooccurence.setObjectName(u"action_cooccurence")
        self.action_create_observations = QAction(MainWindow)
        self.action_create_observations.setObjectName(u"action_create_observations")
        self.actionConfigure_tvevents_columns = QAction(MainWindow)
        self.actionConfigure_tvevents_columns.setObjectName(u"actionConfigure_tvevents_columns")
        self.actionAdd_frame_indexes = QAction(MainWindow)
        self.actionAdd_frame_indexes.setObjectName(u"actionAdd_frame_indexes")
        self.action_load_plugins = QAction(MainWindow)
        self.action_load_plugins.setObjectName(u"action_load_plugins")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.horizontalLayout_2 = QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.w_logo = QWidget(self.centralwidget)
        self.w_logo.setObjectName(u"w_logo")
        self.verticalLayout_8 = QVBoxLayout(self.w_logo)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.verticalLayout_6 = QVBoxLayout()
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.lbLogoBoris = QLabel(self.w_logo)
        self.lbLogoBoris.setObjectName(u"lbLogoBoris")
        self.lbLogoBoris.setScaledContents(False)
        self.lbLogoBoris.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.verticalLayout_6.addWidget(self.lbLogoBoris)

        self.lbLogoUnito = QLabel(self.w_logo)
        self.lbLogoUnito.setObjectName(u"lbLogoUnito")
        self.lbLogoUnito.setTextFormat(Qt.TextFormat.AutoText)
        self.lbLogoUnito.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.lbLogoUnito.setWordWrap(True)

        self.verticalLayout_6.addWidget(self.lbLogoUnito)


        self.verticalLayout_8.addLayout(self.verticalLayout_6)


        self.verticalLayout_3.addWidget(self.w_logo)

        self.w_obs_info = QWidget(self.centralwidget)
        self.w_obs_info.setObjectName(u"w_obs_info")
        self.w_obs_info.setEnabled(True)
        self.verticalLayout_10 = QVBoxLayout(self.w_obs_info)
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.verticalLayout_9 = QVBoxLayout()
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.lb_player_status = QLabel(self.w_obs_info)
        self.lb_player_status.setObjectName(u"lb_player_status")

        self.verticalLayout_9.addWidget(self.lb_player_status)

        self.lb_current_media_time = QLabel(self.w_obs_info)
        self.lb_current_media_time.setObjectName(u"lb_current_media_time")
        self.lb_current_media_time.setWordWrap(True)

        self.verticalLayout_9.addWidget(self.lb_current_media_time)

        self.lb_video_info = QLabel(self.w_obs_info)
        self.lb_video_info.setObjectName(u"lb_video_info")

        self.verticalLayout_9.addWidget(self.lb_video_info)

        self.lb_zoom_level = QLabel(self.w_obs_info)
        self.lb_zoom_level.setObjectName(u"lb_zoom_level")

        self.verticalLayout_9.addWidget(self.lb_zoom_level)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_9.addItem(self.verticalSpacer_2)

        self.lbFocalSubject = QLabel(self.w_obs_info)
        self.lbFocalSubject.setObjectName(u"lbFocalSubject")
        self.lbFocalSubject.setWordWrap(True)

        self.verticalLayout_9.addWidget(self.lbFocalSubject)

        self.lbCurrentStates = QLabel(self.w_obs_info)
        self.lbCurrentStates.setObjectName(u"lbCurrentStates")
        self.lbCurrentStates.setWordWrap(True)

        self.verticalLayout_9.addWidget(self.lbCurrentStates)


        self.verticalLayout_10.addLayout(self.verticalLayout_9)

        self.w_live = QWidget(self.w_obs_info)
        self.w_live.setObjectName(u"w_live")
        self.verticalLayout_11 = QVBoxLayout(self.w_live)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.pb_live_obs = QPushButton(self.w_live)
        self.pb_live_obs.setObjectName(u"pb_live_obs")

        self.verticalLayout_11.addWidget(self.pb_live_obs)


        self.verticalLayout_10.addWidget(self.w_live)


        self.verticalLayout_3.addWidget(self.w_obs_info)


        self.horizontalLayout.addLayout(self.verticalLayout_3)


        self.horizontalLayout_2.addLayout(self.horizontalLayout)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1509, 22))
        self.menuHelp = QMenu(self.menubar)
        self.menuHelp.setObjectName(u"menuHelp")
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        self.menuRecent_projects = QMenu(self.menuFile)
        self.menuRecent_projects.setObjectName(u"menuRecent_projects")
        self.menuImport_project = QMenu(self.menuFile)
        self.menuImport_project.setObjectName(u"menuImport_project")
        self.menuMedia_file_Images_directories = QMenu(self.menuFile)
        self.menuMedia_file_Images_directories.setObjectName(u"menuMedia_file_Images_directories")
        self.menuSet_paths_relative_to_project_dir = QMenu(self.menuMedia_file_Images_directories)
        self.menuSet_paths_relative_to_project_dir.setObjectName(u"menuSet_paths_relative_to_project_dir")
        self.menuRemove_paths = QMenu(self.menuMedia_file_Images_directories)
        self.menuRemove_paths.setObjectName(u"menuRemove_paths")
        self.menuObservations = QMenu(self.menubar)
        self.menuObservations.setObjectName(u"menuObservations")
        self.menuCreate_transitions_matrix = QMenu(self.menuObservations)
        self.menuCreate_transitions_matrix.setObjectName(u"menuCreate_transitions_matrix")
        self.menuExport_events = QMenu(self.menuObservations)
        self.menuExport_events.setObjectName(u"menuExport_events")
        self.menuas_behavioural_sequences = QMenu(self.menuExport_events)
        self.menuas_behavioural_sequences.setObjectName(u"menuas_behavioural_sequences")
        self.menuAnalyze = QMenu(self.menubar)
        self.menuAnalyze.setObjectName(u"menuAnalyze")
        self.menuPlot_events = QMenu(self.menuAnalyze)
        self.menuPlot_events.setObjectName(u"menuPlot_events")
        self.menuInter_rater_reliability = QMenu(self.menuAnalyze)
        self.menuInter_rater_reliability.setObjectName(u"menuInter_rater_reliability")
        self.menuSimilarities = QMenu(self.menuAnalyze)
        self.menuSimilarities.setObjectName(u"menuSimilarities")
        self.menu_plugins = QMenu(self.menuAnalyze)
        self.menu_plugins.setObjectName(u"menu_plugins")
        self.menuPlayback = QMenu(self.menubar)
        self.menuPlayback.setObjectName(u"menuPlayback")
        self.menuImage_overlay_on_video_2 = QMenu(self.menuPlayback)
        self.menuImage_overlay_on_video_2.setObjectName(u"menuImage_overlay_on_video_2")
        self.menuTools = QMenu(self.menubar)
        self.menuTools.setObjectName(u"menuTools")
        self.menuTransitions_flow_diagram = QMenu(self.menuTools)
        self.menuTransitions_flow_diagram.setObjectName(u"menuTransitions_flow_diagram")
        self.menu_coding_map_creator = QMenu(self.menuTools)
        self.menu_coding_map_creator.setObjectName(u"menu_coding_map_creator")
        self.menuMedia_file = QMenu(self.menuTools)
        self.menuMedia_file.setObjectName(u"menuMedia_file")
        MainWindow.setMenuBar(self.menubar)
        self.toolBar = QToolBar(MainWindow)
        self.toolBar.setObjectName(u"toolBar")
        self.toolBar.setEnabled(True)
        MainWindow.addToolBar(Qt.ToolBarArea.TopToolBarArea, self.toolBar)
        self.dwEthogram = QDockWidget(MainWindow)
        self.dwEthogram.setObjectName(u"dwEthogram")
        self.dwEthogram.setFloating(False)
        self.dwEthogram.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetFloatable|QDockWidget.DockWidgetFeature.DockWidgetMovable)
        self.dockWidgetContents_3 = QWidget()
        self.dockWidgetContents_3.setObjectName(u"dockWidgetContents_3")
        self.verticalLayout_5 = QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.twEthogram = QTableWidget(self.dockWidgetContents_3)
        if (self.twEthogram.columnCount() < 8):
            self.twEthogram.setColumnCount(8)
        __qtablewidgetitem = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(2, __qtablewidgetitem2)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(3, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(4, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(5, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(6, __qtablewidgetitem6)
        __qtablewidgetitem7 = QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(7, __qtablewidgetitem7)
        self.twEthogram.setObjectName(u"twEthogram")
        self.twEthogram.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.twEthogram.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.twEthogram.setAlternatingRowColors(True)
        self.twEthogram.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
        self.twEthogram.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.twEthogram.setSortingEnabled(False)

        self.verticalLayout_4.addWidget(self.twEthogram)


        self.verticalLayout_5.addLayout(self.verticalLayout_4)

        self.dwEthogram.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, self.dwEthogram)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        self.statusbar.setSizeGripEnabled(True)
        MainWindow.setStatusBar(self.statusbar)
        self.dwEvents = QDockWidget(MainWindow)
        self.dwEvents.setObjectName(u"dwEvents")
        self.dwEvents.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.dwEvents.setFloating(False)
        self.dwEvents.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetFloatable|QDockWidget.DockWidgetFeature.DockWidgetMovable)
        self.dockWidgetContents_2 = QWidget()
        self.dockWidgetContents_2.setObjectName(u"dockWidgetContents_2")
        self.verticalLayout = QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.splitter = QSplitter(self.dockWidgetContents_2)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Orientation.Vertical)
        self.tv_events = QTableView(self.splitter)
        self.tv_events.setObjectName(u"tv_events")
        self.tv_events.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.tv_events.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.splitter.addWidget(self.tv_events)

        self.verticalLayout.addWidget(self.splitter)

        self.dwEvents.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(Qt.DockWidgetArea.RightDockWidgetArea, self.dwEvents)
        self.dwSubjects = QDockWidget(MainWindow)
        self.dwSubjects.setObjectName(u"dwSubjects")
        self.dwSubjects.setFloating(False)
        self.dwSubjects.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetFloatable|QDockWidget.DockWidgetFeature.DockWidgetMovable)
        self.dockWidgetContents = QWidget()
        self.dockWidgetContents.setObjectName(u"dockWidgetContents")
        self.verticalLayout_2 = QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.twSubjects = QTableWidget(self.dockWidgetContents)
        if (self.twSubjects.columnCount() < 4):
            self.twSubjects.setColumnCount(4)
        __qtablewidgetitem8 = QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(0, __qtablewidgetitem8)
        __qtablewidgetitem9 = QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(1, __qtablewidgetitem9)
        __qtablewidgetitem10 = QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(2, __qtablewidgetitem10)
        __qtablewidgetitem11 = QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(3, __qtablewidgetitem11)
        self.twSubjects.setObjectName(u"twSubjects")
        self.twSubjects.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.twSubjects.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.twSubjects.setAlternatingRowColors(True)
        self.twSubjects.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)

        self.verticalLayout_2.addWidget(self.twSubjects)

        self.dwSubjects.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, self.dwSubjects)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuObservations.menuAction())
        self.menubar.addAction(self.menuPlayback.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuAnalyze.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.menuHelp.addAction(self.actionUser_guide)
        self.menuHelp.addAction(self.actionCheckUpdate)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menuFile.addAction(self.actionNew_project)
        self.menuFile.addAction(self.actionOpen_project)
        self.menuFile.addAction(self.menuRecent_projects.menuAction())
        self.menuFile.addAction(self.menuImport_project.menuAction())
        self.menuFile.addAction(self.actionEdit_project)
        self.menuFile.addAction(self.actionSave_project)
        self.menuFile.addAction(self.actionSave_project_as)
        self.menuFile.addAction(self.actionExport_project)
        self.menuFile.addAction(self.actionClose_project)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionCheck_project)
        self.menuFile.addAction(self.menuMedia_file_Images_directories.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuRecent_projects.addSeparator()
        self.menuImport_project.addAction(self.actionNoldus_Observer_template)
        self.menuMedia_file_Images_directories.addAction(self.actionRemove_path_from_media_files)
        self.menuMedia_file_Images_directories.addAction(self.menuSet_paths_relative_to_project_dir.menuAction())
        self.menuMedia_file_Images_directories.addAction(self.menuRemove_paths.menuAction())
        self.menuSet_paths_relative_to_project_dir.addAction(self.action_media_file_and_images_directories_relative_path)
        self.menuSet_paths_relative_to_project_dir.addAction(self.action_data_files_relative_path)
        self.menuRemove_paths.addAction(self.action_remove_media_files_and_images_directories_path)
        self.menuRemove_paths.addAction(self.action_remove_data_files_path)
        self.menuObservations.addAction(self.actionNew_observation)
        self.menuObservations.addAction(self.actionOpen_observation)
        self.menuObservations.addAction(self.actionView_observation)
        self.menuObservations.addAction(self.actionEdit_observation_2)
        self.menuObservations.addAction(self.actionObservationsList)
        self.menuObservations.addAction(self.actionOpen_observation_2)
        self.menuObservations.addAction(self.actionEdit_observation)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionClose_observation)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.action_create_observations)
        self.menuObservations.addAction(self.actionRemove_observations)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionLoad_observations_file)
        self.menuObservations.addAction(self.actionExport_observations_list)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionAdd_event)
        self.menuObservations.addAction(self.actionEdit_selected_events)
        self.menuObservations.addAction(self.actionEdit_event_time)
        self.menuObservations.addAction(self.actionUndo)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionExplore_project)
        self.menuObservations.addAction(self.actionFind_events)
        self.menuObservations.addAction(self.actionFind_replace_events)
        self.menuObservations.addAction(self.actionCheckStateEvents)
        self.menuObservations.addAction(self.actionClose_unpaired_events)
        self.menuObservations.addAction(self.actionAdd_frame_indexes)
        self.menuObservations.addAction(self.actionSelect_observations)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionDelete_selected_events)
        self.menuObservations.addAction(self.actionDelete_all_events)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuExport_events.menuAction())
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuCreate_subtitles_2)
        self.menuObservations.addAction(self.actionExtract_events_from_media_files)
        self.menuObservations.addAction(self.actionExtract_frames_from_media_files)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuCreate_transitions_matrix.menuAction())
        self.menuCreate_transitions_matrix.addAction(self.actionAll_transitions)
        self.menuCreate_transitions_matrix.addAction(self.actionFrequencies_of_transitions_after_behaviors)
        self.menuCreate_transitions_matrix.addAction(self.actionNumber_of_transitions)
        self.menuExport_events.addAction(self.actionExportEvents_2)
        self.menuExport_events.addAction(self.actionExport_aggregated_events)
        self.menuExport_events.addAction(self.menuas_behavioural_sequences.menuAction())
        self.menuExport_events.addAction(self.actionExport_events_as_Praat_TextGrid)
        self.menuExport_events.addAction(self.actionJWatcher)
        self.menuExport_events.addAction(self.action_behavior_binary_table)
        self.menuas_behavioural_sequences.addAction(self.actionseparated_subjects)
        self.menuas_behavioural_sequences.addAction(self.actiongrouped_subjects)
        self.menuAnalyze.addAction(self.menu_plugins.menuAction())
        self.menuAnalyze.addAction(self.actionTime_budget)
        self.menuAnalyze.addAction(self.actionTime_budget_by_behaviors_category)
        self.menuAnalyze.addAction(self.actionTime_budget_report)
        self.menuAnalyze.addAction(self.actionSynthetic_binned_time_budget)
        self.menuAnalyze.addAction(self.menuPlot_events.menuAction())
        self.menuAnalyze.addAction(self.menuInter_rater_reliability.menuAction())
        self.menuAnalyze.addAction(self.menuSimilarities.menuAction())
        self.menuAnalyze.addAction(self.action_cooccurence)
        self.menuAnalyze.addAction(self.action_advanced_event_filtering)
        self.menuAnalyze.addAction(self.action_latency)
        self.menuPlot_events.addAction(self.actionPlot_events2)
        self.menuPlot_events.addAction(self.actionBehavior_bar_plot)
        self.menuInter_rater_reliability.addAction(self.actionCohen_s_kappa)
        self.menuSimilarities.addAction(self.actionNeedleman_Wunsch)
        self.menu_plugins.addAction(self.action_load_plugins)
        self.menuPlayback.addAction(self.actionPlay)
        self.menuPlayback.addAction(self.actionPrevious)
        self.menuPlayback.addAction(self.actionNext)
        self.menuPlayback.addAction(self.actionJumpForward)
        self.menuPlayback.addAction(self.actionJumpBackward)
        self.menuPlayback.addAction(self.actionJumpTo)
        self.menuPlayback.addAction(self.action_deinterlace)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionZoom_level)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionRotate_current_video)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionDisplay_subtitles)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.menuImage_overlay_on_video_2.menuAction())
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionVideo_equalizer)
        self.menuImage_overlay_on_video_2.addAction(self.actionAdd_image_overlay_on_video)
        self.menuImage_overlay_on_video_2.addAction(self.actionRemove_image_overlay)
        self.menuTools.addAction(self.menuMedia_file.menuAction())
        self.menuTools.addAction(self.action_block_dockwidgets)
        self.menuTools.addAction(self.actionPlot_events_in_real_time)
        self.menuTools.addAction(self.actionShow_spectrogram)
        self.menuTools.addAction(self.actionShow_the_sound_waveform)
        self.menuTools.addAction(self.actionShow_data_files)
        self.menuTools.addAction(self.action_geometric_measurements)
        self.menuTools.addAction(self.actionBehaviors_coding_map)
        self.menuTools.addAction(self.actionCoding_pad)
        self.menuTools.addAction(self.actionSubjects_pad)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.menu_coding_map_creator.menuAction())
        self.menuTools.addAction(self.menuTransitions_flow_diagram.menuAction())
        self.menuTransitions_flow_diagram.addAction(self.actionCreate_transitions_flow_diagram)
        self.menuTransitions_flow_diagram.addAction(self.actionCreate_transitions_flow_diagram_2)
        self.menu_coding_map_creator.addAction(self.action_create_behaviors_coding_map)
        self.menu_coding_map_creator.addAction(self.action_create_modifiers_coding_map)
        self.menuMedia_file.addAction(self.actionMedia_file_information_2)
        self.menuMedia_file.addAction(self.actionRecode_resize_video)
        self.menuMedia_file.addAction(self.actionRotate_video)
        self.menuMedia_file.addAction(self.actionMerge_media_files)
        self.toolBar.addAction(self.action_obs_list)
        self.toolBar.addAction(self.actionPlay)
        self.toolBar.addAction(self.actionReset)
        self.toolBar.addAction(self.actionJumpBackward)
        self.toolBar.addAction(self.actionJumpForward)
        self.toolBar.addAction(self.actionNormalSpeed)
        self.toolBar.addAction(self.actionFaster)
        self.toolBar.addAction(self.actionSlower)
        self.toolBar.addAction(self.actionPrevious)
        self.toolBar.addAction(self.actionNext)
        self.toolBar.addAction(self.actionSnapshot)
        self.toolBar.addAction(self.actionFrame_by_frame)
        self.toolBar.addAction(self.actionFrame_backward)
        self.toolBar.addAction(self.actionFrame_forward)
        self.toolBar.addAction(self.actionCloseObs)
        self.toolBar.addAction(self.actionPlot_events_in_real_time)
        self.toolBar.addAction(self.actionCurrent_Time_Budget)
        self.toolBar.addAction(self.actionPlot_current_observation)
        self.toolBar.addAction(self.actionPlot_current_time_budget)
        self.toolBar.addAction(self.action_geometric_measurements)
        self.toolBar.addAction(self.actionFind_in_current_obs)
        self.toolBar.addAction(self.actionExplore_project)
        self.toolBar.addAction(self.actionPreferences)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"BORIS", None))
        self.actionDocumentation.setText(QCoreApplication.translate("MainWindow", u"Documentation", None))
        self.actionAbout.setText(QCoreApplication.translate("MainWindow", u"About", None))
        self.actionQuit.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
        self.actionPause.setText(QCoreApplication.translate("MainWindow", u"Pause", None))
#if QT_CONFIG(tooltip)
        self.actionPause.setToolTip(QCoreApplication.translate("MainWindow", u"Pause", None))
#endif // QT_CONFIG(tooltip)
        self.actionPlay.setText(QCoreApplication.translate("MainWindow", u"Play", None))
        self.actionOpen_video_file.setText(QCoreApplication.translate("MainWindow", u"Open media file", None))
        self.actionReset.setText(QCoreApplication.translate("MainWindow", u"Reset", None))
        self.actionFaster.setText(QCoreApplication.translate("MainWindow", u"Faster", None))
        self.actionSlower.setText(QCoreApplication.translate("MainWindow", u"Slower", None))
        self.actionJumpForward.setText(QCoreApplication.translate("MainWindow", u"Jump forward", None))
#if QT_CONFIG(shortcut)
        self.actionJumpForward.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+F", None))
#endif // QT_CONFIG(shortcut)
        self.actionLoad_configuration.setText(QCoreApplication.translate("MainWindow", u"Load configuration", None))
        self.actionDelete_selected_events.setText(QCoreApplication.translate("MainWindow", u"Delete selected events", None))
        self.actionDelete_all_events.setText(QCoreApplication.translate("MainWindow", u"Delete all events", None))
        self.actionSort_observations.setText(QCoreApplication.translate("MainWindow", u"Sort events", None))
        self.actionLoad_observations_file.setText(QCoreApplication.translate("MainWindow", u"Import observations", None))
        self.actionSelect_observations.setText(QCoreApplication.translate("MainWindow", u"Select events from interval", None))
        self.actionConfigure_states_and_events.setText(QCoreApplication.translate("MainWindow", u"Configure states and events", None))
        self.actionEdit_event.setText(QCoreApplication.translate("MainWindow", u"Edit event", None))
        self.actionLoad_configuration_file.setText(QCoreApplication.translate("MainWindow", u"Load state and events configuration file", None))
        self.actionMedia_file_information.setText(QCoreApplication.translate("MainWindow", u"Media file information", None))
        self.actionStart_live_observation.setText(QCoreApplication.translate("MainWindow", u"Start observation without media file", None))
        self.actionNew_project.setText(QCoreApplication.translate("MainWindow", u"New project", None))
        self.actionTime_budget.setText(QCoreApplication.translate("MainWindow", u"Time budget", None))
        self.actionSave_project.setText(QCoreApplication.translate("MainWindow", u"Save project", None))
#if QT_CONFIG(shortcut)
        self.actionSave_project.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+S", None))
#endif // QT_CONFIG(shortcut)
        self.actionOpen_project.setText(QCoreApplication.translate("MainWindow", u"Open project", None))
        self.actionSet_offset.setText(QCoreApplication.translate("MainWindow", u"Set time offset", None))
        self.actionEdit_project.setText(QCoreApplication.translate("MainWindow", u"Edit project", None))
        self.actionSave_project_as.setText(QCoreApplication.translate("MainWindow", u"Save project as ...", None))
        self.actionPreferences.setText(QCoreApplication.translate("MainWindow", u"Preferences", None))
        self.actionNew_observation.setText(QCoreApplication.translate("MainWindow", u"New observation", None))
#if QT_CONFIG(shortcut)
        self.actionNew_observation.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+N", None))
#endif // QT_CONFIG(shortcut)
        self.actionSave_observation.setText(QCoreApplication.translate("MainWindow", u"Save current observation", None))
        self.actionClose_observation.setText(QCoreApplication.translate("MainWindow", u"Close observation", None))
#if QT_CONFIG(shortcut)
        self.actionClose_observation.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Q", None))
#endif // QT_CONFIG(shortcut)
        self.actionEdit_current_observation.setText(QCoreApplication.translate("MainWindow", u"Edit current observation", None))
        self.actionOpen_observation_2.setText(QCoreApplication.translate("MainWindow", u"Open observation", None))
        self.actionAdd_event.setText(QCoreApplication.translate("MainWindow", u"Add event", None))
#if QT_CONFIG(shortcut)
        self.actionAdd_event.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+A", None))
#endif // QT_CONFIG(shortcut)
        self.actionDeselectCurrentSubject.setText(QCoreApplication.translate("MainWindow", u"Deselect current subject", None))
        self.actionNext.setText(QCoreApplication.translate("MainWindow", u"Next", None))
#if QT_CONFIG(tooltip)
        self.actionNext.setToolTip(QCoreApplication.translate("MainWindow", u"Next media file", None))
#endif // QT_CONFIG(tooltip)
        self.actionPrevious.setText(QCoreApplication.translate("MainWindow", u"Previous", None))
#if QT_CONFIG(tooltip)
        self.actionPrevious.setToolTip(QCoreApplication.translate("MainWindow", u"Previous media file", None))
#endif // QT_CONFIG(tooltip)
        self.actionJumpTo.setText(QCoreApplication.translate("MainWindow", u"Jump to specific time", None))
        self.actionJumpBackward.setText(QCoreApplication.translate("MainWindow", u"Jump backward", None))
#if QT_CONFIG(tooltip)
        self.actionJumpBackward.setToolTip(QCoreApplication.translate("MainWindow", u"Jump backward", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionJumpBackward.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+B", None))
#endif // QT_CONFIG(shortcut)
        self.actionEdit_observation.setText(QCoreApplication.translate("MainWindow", u"Edit observation", None))
        self.actionCheckUpdate.setText(QCoreApplication.translate("MainWindow", u"Check for updates and news", None))
        self.actionClose_project.setText(QCoreApplication.translate("MainWindow", u"Close project", None))
        self.actionObservationsList.setText(QCoreApplication.translate("MainWindow", u"Observations list", None))
#if QT_CONFIG(shortcut)
        self.actionObservationsList.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+L", None))
#endif // QT_CONFIG(shortcut)
        self.actionNormalSpeed.setText(QCoreApplication.translate("MainWindow", u"Normal speed", None))
        self.actionSnapshot.setText(QCoreApplication.translate("MainWindow", u"Snapshot", None))
        self.actionFrame_by_frame.setText(QCoreApplication.translate("MainWindow", u"Frame by frame", None))
        self.actionAggregatedEventsTabularFormat.setText(QCoreApplication.translate("MainWindow", u"Tab Separated Values (tsv)", None))
        self.actionOpen_observation.setText(QCoreApplication.translate("MainWindow", u"Start observation", None))
#if QT_CONFIG(shortcut)
        self.actionOpen_observation.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+O", None))
#endif // QT_CONFIG(shortcut)
        self.actionAaaa.setText(QCoreApplication.translate("MainWindow", u"aaaa", None))
        self.menuCreate_subtitles_2.setText(QCoreApplication.translate("MainWindow", u"Create subtitles", None))
        self.actionUser_guide.setText(QCoreApplication.translate("MainWindow", u"User guide", None))
        self.actionEdit_observation_2.setText(QCoreApplication.translate("MainWindow", u"Edit observation", None))
#if QT_CONFIG(shortcut)
        self.actionEdit_observation_2.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+E", None))
#endif // QT_CONFIG(shortcut)
        self.actionCheckStateEvents.setText(QCoreApplication.translate("MainWindow", u"Check state events", None))
        self.actionRunEventOutside.setText(QCoreApplication.translate("MainWindow", u"Run external program with selected event(s)", None))
        self.actionEdit_selected_events.setText(QCoreApplication.translate("MainWindow", u"Edit selected event(s)", None))
        self.actionShow_spectrogram.setText(QCoreApplication.translate("MainWindow", u"Show the sound spectrogram", None))
        self.actionExport_events_as_Praat_TextGrid.setText(QCoreApplication.translate("MainWindow", u"as Praat TextGrid", None))
        self.actionExtract_events_from_media_files.setText(QCoreApplication.translate("MainWindow", u"Extract sequences from media files", None))
        self.action_geometric_measurements.setText(QCoreApplication.translate("MainWindow", u"Geometric measurement", None))
        self.actionFrame_forward.setText(QCoreApplication.translate("MainWindow", u"Frame forward", None))
        self.actionFrame_backward.setText(QCoreApplication.translate("MainWindow", u"frame backward", None))
        self.actionFilterBehaviors.setText(QCoreApplication.translate("MainWindow", u"Filter behaviors", None))
        self.actionShowAllBehaviors.setText(QCoreApplication.translate("MainWindow", u"Show all behaviors", None))
#if QT_CONFIG(tooltip)
        self.actionShowAllBehaviors.setToolTip(QCoreApplication.translate("MainWindow", u"Show all behaviors", None))
#endif // QT_CONFIG(tooltip)
        self.actionExport_aggregated_events.setText(QCoreApplication.translate("MainWindow", u"Aggregated events", None))
        self.actionCoding_pad.setText(QCoreApplication.translate("MainWindow", u"Coding pad", None))
        self.actionTime_budget_by_behaviors_category.setText(QCoreApplication.translate("MainWindow", u"Time budget by behavioral category", None))
        self.actionExport_events_as_SDIS_file.setText(QCoreApplication.translate("MainWindow", u"Export events as SDIS file", None))
        self.actionRecode_resize_video.setText(QCoreApplication.translate("MainWindow", u"Re-encode/resize video", None))
        self.actionMedia_file_information_2.setText(QCoreApplication.translate("MainWindow", u"Media file information", None))
        self.actionCreate_transitions_flow_diagram.setText(QCoreApplication.translate("MainWindow", u"Create transitions DOT script", None))
        self.actionCreate_transitions_flow_diagram_2.setText(QCoreApplication.translate("MainWindow", u"Create transitions flow diagram", None))
        self.actionAll_transitions.setText(QCoreApplication.translate("MainWindow", u"Frequencies of transitions", None))
        self.actionNumber_of_transitions.setText(QCoreApplication.translate("MainWindow", u"Number of transitions", None))
        self.actionFrequencies_of_transitions_after_behaviors.setText(QCoreApplication.translate("MainWindow", u"Frequencies of transitions after behaviors", None))
        self.actionFind_replace_events.setText(QCoreApplication.translate("MainWindow", u"Find/replace in events", None))
        self.actionFind_events.setText(QCoreApplication.translate("MainWindow", u"Find in events", None))
#if QT_CONFIG(tooltip)
        self.actionFind_events.setToolTip(QCoreApplication.translate("MainWindow", u"Find", None))
#endif // QT_CONFIG(tooltip)
        self.actionHow_to_cite_BORIS.setText(QCoreApplication.translate("MainWindow", u"How to cite BORIS", None))
        self.actionZoom1_1_1.setText(QCoreApplication.translate("MainWindow", u"1:1 Original", None))
        self.actionZoom1_2_1.setText(QCoreApplication.translate("MainWindow", u"2:1 Double", None))
        self.actionZoom1_1_2.setText(QCoreApplication.translate("MainWindow", u"1:2 Half", None))
        self.actionZoom1_1_4.setText(QCoreApplication.translate("MainWindow", u"1:4 Quarter", None))
        self.actionZoom1_fitwindow.setText(QCoreApplication.translate("MainWindow", u"Fit to window", None))
        self.actionZoom2_fitwindow.setText(QCoreApplication.translate("MainWindow", u"Fit to window", None))
        self.actionZoom2_1_4.setText(QCoreApplication.translate("MainWindow", u"1:4 Quarter", None))
        self.actionZoom2_1_2.setText(QCoreApplication.translate("MainWindow", u"1:2 Half", None))
        self.actionZoom2_1_1.setText(QCoreApplication.translate("MainWindow", u"1:1 Original", None))
        self.actionZoom2_2_1.setText(QCoreApplication.translate("MainWindow", u"2:1 Double", None))
        self.actionFilterSubjects.setText(QCoreApplication.translate("MainWindow", u"Filter subjects", None))
#if QT_CONFIG(tooltip)
        self.actionFilterSubjects.setToolTip(QCoreApplication.translate("MainWindow", u"Filter subjects", None))
#endif // QT_CONFIG(tooltip)
        self.actionShowAllSubjects.setText(QCoreApplication.translate("MainWindow", u"Show all subjects", None))
#if QT_CONFIG(tooltip)
        self.actionShowAllSubjects.setToolTip(QCoreApplication.translate("MainWindow", u"Show all subjects", None))
#endif // QT_CONFIG(tooltip)
        self.actionSend_project.setText(QCoreApplication.translate("MainWindow", u"Project server", None))
        self.actionViewBehavior.setText(QCoreApplication.translate("MainWindow", u"View behavior", None))
#if QT_CONFIG(tooltip)
        self.actionViewBehavior.setToolTip(QCoreApplication.translate("MainWindow", u"View behavior", None))
#endif // QT_CONFIG(tooltip)
        self.actionBehaviors_coding_map.setText(QCoreApplication.translate("MainWindow", u"Behaviors coding map", None))
        self.action_create_behaviors_coding_map.setText(QCoreApplication.translate("MainWindow", u"for behaviors", None))
        self.action_create_modifiers_coding_map.setText(QCoreApplication.translate("MainWindow", u"for modifiers", None))
        self.actionPlot_events1.setText(QCoreApplication.translate("MainWindow", u"Plot events (back compatibility)", None))
        self.actionPlot_events2.setText(QCoreApplication.translate("MainWindow", u"Plot events", None))
        self.actionCohen_s_kappa.setText(QCoreApplication.translate("MainWindow", u"Cohen's kappa (time-unit)", None))
        self.actionTime_budget_report.setText(QCoreApplication.translate("MainWindow", u"Synthetic time budget", None))
        self.actionCheckStateEventsSingleObs.setText(QCoreApplication.translate("MainWindow", u"Check state events", None))
        self.actionSubjects_pad.setText(QCoreApplication.translate("MainWindow", u"Subjects pad", None))
        self.actionShow_data_files.setText(QCoreApplication.translate("MainWindow", u"Show data files", None))
        self.actionConverters_manager.setText(QCoreApplication.translate("MainWindow", u"Time converters manager", None))
        self.actionView_observation.setText(QCoreApplication.translate("MainWindow", u"View observation", None))
        self.actionRemove_path_from_media_files.setText(QCoreApplication.translate("MainWindow", u"Remove paths", None))
        self.actionCheck_project.setText(QCoreApplication.translate("MainWindow", u"Check project integrity", None))
        self.actionBehavior_bar_plot.setText(QCoreApplication.translate("MainWindow", u"Plot time budget", None))
        self.action_obs_list.setText(QCoreApplication.translate("MainWindow", u"Obs list", None))
#if QT_CONFIG(tooltip)
        self.action_obs_list.setToolTip(QCoreApplication.translate("MainWindow", u"Observations list", None))
#endif // QT_CONFIG(tooltip)
        self.actionTest.setText(QCoreApplication.translate("MainWindow", u"test", None))
        self.actionTest_stb2.setText(QCoreApplication.translate("MainWindow", u"test stb2", None))
        self.actionClose_unpaired_events.setText(QCoreApplication.translate("MainWindow", u"Fix unpaired events", None))
#if QT_CONFIG(shortcut)
        self.actionClose_unpaired_events.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+U", None))
#endif // QT_CONFIG(shortcut)
        self.actionRotate_video.setText(QCoreApplication.translate("MainWindow", u"Rotate video", None))
        self.actionExport_observations_list.setText(QCoreApplication.translate("MainWindow", u"Export observations list", None))
        self.actionExportEvents_2.setText(QCoreApplication.translate("MainWindow", u"Tabular events", None))
        self.actionJWatcher.setText(QCoreApplication.translate("MainWindow", u"for analysis with JWatcher", None))
        self.actionCloseObs.setText(QCoreApplication.translate("MainWindow", u"Close observation", None))
        self.actionFilter_events.setText(QCoreApplication.translate("MainWindow", u"Filter events", None))
        self.actionShow_all_events.setText(QCoreApplication.translate("MainWindow", u"Show all events", None))
#if QT_CONFIG(tooltip)
        self.actionShow_all_events.setToolTip(QCoreApplication.translate("MainWindow", u"Show all events", None))
#endif // QT_CONFIG(tooltip)
        self.actionEdit_event_time.setText(QCoreApplication.translate("MainWindow", u"Shift time of selected event(s)", None))
        self.actionCopy_events.setText(QCoreApplication.translate("MainWindow", u"Copy events", None))
        self.actionPaste_events.setText(QCoreApplication.translate("MainWindow", u"Paste events", None))
        self.actionNoldus_Observer_template.setText(QCoreApplication.translate("MainWindow", u"Noldus The Observer XT (OTX, OTB, ODX)", None))
        self.actionExplore_project.setText(QCoreApplication.translate("MainWindow", u"Explore project", None))
        self.actionShow_the_sound_waveform.setText(QCoreApplication.translate("MainWindow", u"Show the sound waveform", None))
        self.actionNeedleman_Wunsch.setText(QCoreApplication.translate("MainWindow", u"Needleman - Wunsch", None))
        self.actionExtract_frames_from_media_files.setText(QCoreApplication.translate("MainWindow", u"Extract frames from media files", None))
        self.action_behavior_binary_table.setText(QCoreApplication.translate("MainWindow", u"as behaviors binary table", None))
        self.actionPlot_current_observation.setText(QCoreApplication.translate("MainWindow", u"Plot current observation", None))
        self.actionFind_in_current_obs.setText(QCoreApplication.translate("MainWindow", u"Find in events", None))
        self.action_advanced_event_filtering.setText(QCoreApplication.translate("MainWindow", u"Advanced event filtering", None))
        self.action_block_dockwidgets.setText(QCoreApplication.translate("MainWindow", u"Lock dockwidgets", None))
        self.actionseparated_subjects.setText(QCoreApplication.translate("MainWindow", u"separated subjects", None))
        self.actiongrouped_subjects.setText(QCoreApplication.translate("MainWindow", u"grouped subjects", None))
        self.actionCurrent_Time_Budget.setText(QCoreApplication.translate("MainWindow", u"Current Time Budget", None))
        self.actionSynthetic_binned_time_budget.setText(QCoreApplication.translate("MainWindow", u"Synthetic time budget with time bin", None))
        self.actionZoom_level.setText(QCoreApplication.translate("MainWindow", u"Zoom level", None))
        self.actionDisplay_subtitles.setText(QCoreApplication.translate("MainWindow", u"Display subtitles", None))
        self.actionAdd_image_overlay_on_video.setText(QCoreApplication.translate("MainWindow", u"Add", None))
        self.actionRemove_image_overlay.setText(QCoreApplication.translate("MainWindow", u"Remove", None))
        self.actionPlot_events_in_real_time.setText(QCoreApplication.translate("MainWindow", u"Plot events in real time", None))
        self.actionVideo_equalizer.setText(QCoreApplication.translate("MainWindow", u"Video equalizer", None))
        self.actionRemove_observations.setText(QCoreApplication.translate("MainWindow", u"Remove observations", None))
        self.action_latency.setText(QCoreApplication.translate("MainWindow", u"Latency", None))
        self.actionUndo.setText(QCoreApplication.translate("MainWindow", u"Undo", None))
        self.actionaaa.setText(QCoreApplication.translate("MainWindow", u"aaa", None))
        self.action_media_file_and_images_directories_relative_path.setText(QCoreApplication.translate("MainWindow", u"for media file and images directories", None))
        self.action_data_files_relative_path.setText(QCoreApplication.translate("MainWindow", u"for data files", None))
        self.action_remove_media_files_and_images_directories_path.setText(QCoreApplication.translate("MainWindow", u"for media file and images directories", None))
        self.action_remove_data_files_path.setText(QCoreApplication.translate("MainWindow", u"for data files", None))
        self.actionPlot_current_time_budget.setText(QCoreApplication.translate("MainWindow", u"Plot current time budget", None))
        self.actionMerge_media_files.setText(QCoreApplication.translate("MainWindow", u"Merge media files", None))
        self.actionExport_project.setText(QCoreApplication.translate("MainWindow", u"Export project", None))
        self.actionConfigure_twEvents_columns.setText(QCoreApplication.translate("MainWindow", u"Configure columns", None))
        self.actionRotate_current_video.setText(QCoreApplication.translate("MainWindow", u"Rotate video", None))
        self.action_deinterlace.setText(QCoreApplication.translate("MainWindow", u"Deinterlace", None))
        self.action_cooccurence.setText(QCoreApplication.translate("MainWindow", u"Co-occurence", None))
        self.action_create_observations.setText(QCoreApplication.translate("MainWindow", u"Create observations", None))
        self.actionConfigure_tvevents_columns.setText(QCoreApplication.translate("MainWindow", u"Configure columns", None))
        self.actionAdd_frame_indexes.setText(QCoreApplication.translate("MainWindow", u"Add frame indexes", None))
        self.action_load_plugins.setText(QCoreApplication.translate("MainWindow", u"Load plugins", None))
        self.lbLogoBoris.setText("")
        self.lbLogoUnito.setText("")
        self.lb_player_status.setText(QCoreApplication.translate("MainWindow", u"lb_player_status", None))
        self.lb_current_media_time.setText(QCoreApplication.translate("MainWindow", u"lb_current_media_time", None))
        self.lb_video_info.setText(QCoreApplication.translate("MainWindow", u"lb_video_info", None))
        self.lb_zoom_level.setText(QCoreApplication.translate("MainWindow", u"lb_zoom_level", None))
        self.lbFocalSubject.setText(QCoreApplication.translate("MainWindow", u"lbFocalSubject", None))
        self.lbCurrentStates.setText(QCoreApplication.translate("MainWindow", u"lbCurrentStates", None))
        self.pb_live_obs.setText(QCoreApplication.translate("MainWindow", u"PushButton", None))
        self.menuHelp.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"Project", None))
        self.menuRecent_projects.setTitle(QCoreApplication.translate("MainWindow", u"Recent projects", None))
        self.menuImport_project.setTitle(QCoreApplication.translate("MainWindow", u"Import project from...", None))
        self.menuMedia_file_Images_directories.setTitle(QCoreApplication.translate("MainWindow", u"External files", None))
        self.menuSet_paths_relative_to_project_dir.setTitle(QCoreApplication.translate("MainWindow", u"Set paths relative to project dir", None))
        self.menuRemove_paths.setTitle(QCoreApplication.translate("MainWindow", u"Remove paths", None))
        self.menuObservations.setTitle(QCoreApplication.translate("MainWindow", u"Observations", None))
        self.menuCreate_transitions_matrix.setTitle(QCoreApplication.translate("MainWindow", u"Create transitions matrix", None))
        self.menuExport_events.setTitle(QCoreApplication.translate("MainWindow", u"Export events", None))
        self.menuas_behavioural_sequences.setTitle(QCoreApplication.translate("MainWindow", u"as behavioural sequences", None))
        self.menuAnalyze.setTitle(QCoreApplication.translate("MainWindow", u"Analysis", None))
        self.menuPlot_events.setTitle(QCoreApplication.translate("MainWindow", u"Plot", None))
        self.menuInter_rater_reliability.setTitle(QCoreApplication.translate("MainWindow", u"Inter-rater reliability", None))
        self.menuSimilarities.setTitle(QCoreApplication.translate("MainWindow", u"Similarities", None))
        self.menu_plugins.setTitle(QCoreApplication.translate("MainWindow", u"Plugins", None))
        self.menuPlayback.setTitle(QCoreApplication.translate("MainWindow", u"Playback", None))
        self.menuImage_overlay_on_video_2.setTitle(QCoreApplication.translate("MainWindow", u"Image overlay on video", None))
        self.menuTools.setTitle(QCoreApplication.translate("MainWindow", u"Tools", None))
        self.menuTransitions_flow_diagram.setTitle(QCoreApplication.translate("MainWindow", u"Transitions flow diagram", None))
        self.menu_coding_map_creator.setTitle(QCoreApplication.translate("MainWindow", u"Create a coding map", None))
        self.menuMedia_file.setTitle(QCoreApplication.translate("MainWindow", u"Media file", None))
        self.toolBar.setWindowTitle(QCoreApplication.translate("MainWindow", u"toolBar", None))
#if QT_CONFIG(tooltip)
        self.toolBar.setToolTip("")
#endif // QT_CONFIG(tooltip)
        self.dwEthogram.setWindowTitle(QCoreApplication.translate("MainWindow", u"Ethogram", None))
        ___qtablewidgetitem = self.twEthogram.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("MainWindow", u"Key", None));
        ___qtablewidgetitem1 = self.twEthogram.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("MainWindow", u"Code", None));
        ___qtablewidgetitem2 = self.twEthogram.horizontalHeaderItem(2)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("MainWindow", u"Type", None));
        ___qtablewidgetitem3 = self.twEthogram.horizontalHeaderItem(3)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("MainWindow", u"Description", None));
        ___qtablewidgetitem4 = self.twEthogram.horizontalHeaderItem(4)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("MainWindow", u"Color", None));
        ___qtablewidgetitem5 = self.twEthogram.horizontalHeaderItem(5)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("MainWindow", u"Category", None));
        ___qtablewidgetitem6 = self.twEthogram.horizontalHeaderItem(6)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("MainWindow", u"Modifiers", None));
        ___qtablewidgetitem7 = self.twEthogram.horizontalHeaderItem(7)
        ___qtablewidgetitem7.setText(QCoreApplication.translate("MainWindow", u"Excluded", None));
        self.dwEvents.setWindowTitle(QCoreApplication.translate("MainWindow", u"Events", None))
        self.dwSubjects.setWindowTitle(QCoreApplication.translate("MainWindow", u"Subjects", None))
        ___qtablewidgetitem8 = self.twSubjects.horizontalHeaderItem(0)
        ___qtablewidgetitem8.setText(QCoreApplication.translate("MainWindow", u"Key", None));
        ___qtablewidgetitem9 = self.twSubjects.horizontalHeaderItem(1)
        ___qtablewidgetitem9.setText(QCoreApplication.translate("MainWindow", u"Name", None));
        ___qtablewidgetitem10 = self.twSubjects.horizontalHeaderItem(2)
        ___qtablewidgetitem10.setText(QCoreApplication.translate("MainWindow", u"Description", None));
        ___qtablewidgetitem11 = self.twSubjects.horizontalHeaderItem(3)
        ___qtablewidgetitem11.setText(QCoreApplication.translate("MainWindow", u"Current state(s)", None));
    # retranslateUi




"""
BORIS plugin

number of occurences of behaviors
"""

import pandas as pd

__version__ = "0.3.0"
__version_date__ = "2025-03-17"
__plugin_name__ = "Number of occurences of behaviors"
__author__ = "Olivier Friard - University of Torino - Italy"


def run(df: pd.DataFrame):
    """
    Calculate the number of occurrences of behaviors by subject.
    """

    df_results: pd.DataFrame = df.groupby(["Subject", "Behavior"])["Behavior"].count().reset_index(name="number of occurences")

    return df_results



import sys
import os
import pytest
import subprocess
from unittest.mock import patch, MagicMock
import logging

# Assuming the function is imported from a module named `media_analysis`
# from media_analysis import accurate_media_analysis

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from boris import utilities
from boris import config


# Mocking the ffprobe_media_analysis function
def mock_ffprobe_media_analysis(ffmpeg_bin, file_name):
    return {
        "duration": 120,
        "duration_ms": 120000,
        "bitrate": 5000,
        "frames_number": 3000,
        "fps": 25,
        "has_video": True,
        "has_audio": True,
    }


# Mocking the time2seconds function
def mock_time2seconds(time_str):
    return 120


# Mocking the dec function
def mock_dec(value):
    return float(value)


@patch("utilities.ffprobe_media_analysis", side_effect=mock_ffprobe_media_analysis)
@patch("utilities.time2seconds", side_effect=mock_time2seconds)
@patch("utilities.dec", side_effect=mock_dec)
def test_accurate_media_analysis_success(mock_dec, mock_time2seconds, mock_ffprobe_media_analysis):
    ffmpeg_bin = "/path/to/ffmpeg"
    file_name = "test_video.mp4"

    result = utilities.accurate_media_analysis(ffmpeg_bin, file_name)

    assert result["duration"] == 120
    assert result["duration_ms"] == 120000
    assert result["bitrate"] == 5000
    assert result["frames_number"] == 3000
    assert result["fps"] == 25
    assert result["has_video"] is True
    assert result["has_audio"] is True


"""

@patch("utilities.ffprobe_media_analysis", return_value={"error": "ffprobe failed"})
@patch("utilities.time2seconds", side_effect=mock_time2seconds)
@patch("utilities.dec", side_effect=mock_dec)
@patch("subprocess.Popen")
def test_accurate_media_analysis_fallback(mock_popen, mock_dec, mock_time2seconds, mock_ffprobe_media_analysis):
    ffmpeg_bin = "/path/to/ffmpeg"
    file_name = "test_video.mp4"

    # Mocking the subprocess.Popen behavior
    mock_process = MagicMock()
    mock_process.communicate.return_value = (
        b"",
        b"Duration: 00:02:00.00, bitrate: 5000 kb/s\nStream #0:0: Video: h264, 1920x1080, 25 fps\nStream #0:1: Audio: aac",
    )
    mock_popen.return_value = mock_process

    result = utilities.accurate_media_analysis(ffmpeg_bin, file_name)

    assert result["duration"] == 120
    assert result["duration_ms"] == 120000
    assert result["bitrate"] == 5000000
    assert result["frames_number"] == 3000
    assert result["fps"] == 25
    assert result["has_video"] is True
    assert result["has_audio"] is True


@patch("utilities.ffprobe_media_analysis", return_value={"error": "ffprobe failed"})
@patch("utilities.time2seconds", side_effect=mock_time2seconds)
@patch("utilities.dec", side_effect=mock_dec)
@patch("subprocess.Popen")
def test_accurate_media_analysis_file_not_found(mock_popen, mock_dec, mock_time2seconds, mock_ffprobe_media_analysis):
    ffmpeg_bin = "/path/to/ffmpeg"
    file_name = "nonexistent_video.mp4"

    # Mocking the subprocess.Popen behavior
    mock_process = MagicMock()
    mock_process.communicate.return_value = (b"", b"No such file or directory")
    mock_popen.return_value = mock_process

    result = utilities.accurate_media_analysis(ffmpeg_bin, file_name)

    assert result["error"] == "No such file or directory"


@patch("utilities.ffprobe_media_analysis", return_value={"error": "ffprobe failed"})
@patch("utilities.time2seconds", side_effect=mock_time2seconds)
@patch("utilities.dec", side_effect=mock_dec)
@patch("subprocess.Popen")
def test_accurate_media_analysis_invalid_media(mock_popen, mock_dec, mock_time2seconds, mock_ffprobe_media_analysis):
    ffmpeg_bin = "/path/to/ffmpeg"
    file_name = "invalid_media.mp4"

    # Mocking the subprocess.Popen behavior
    mock_process = MagicMock()
    mock_process.communicate.return_value = (b"", b"Invalid data found when processing input")
    mock_popen.return_value = mock_process

    result = utilities.accurate_media_analysis(ffmpeg_bin, file_name)

    assert result["error"] == "This file does not seem to be a media file"


"""



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.

"""

import logging
from . import config as cfg
from . import dialog


def add_image_overlay(self) -> None:
    """
    add an image overlay on video from an image
    """

    logging.debug("function add_image_overlay")

    try:
        w = dialog.Video_overlay_dialog()
        items = [f"Player #{i + 1}" for i, _ in enumerate(self.dw_player)]
        w.cb_player.addItems(items)
        if not w.exec_():
            return

        idx = w.cb_player.currentIndex()

        if cfg.OVERLAY not in self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO]:
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO][cfg.OVERLAY] = {}
        self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO][cfg.OVERLAY][str(idx + 1)] = {
            "file name": w.le_file_path.text(),
            "overlay position": w.le_overlay_position.text(),
            "transparency": w.sb_overlay_transparency.value(),
        }
        self.overlays[idx] = self.dw_player[idx].player.create_image_overlay()
        self.project_changed()
        self.resize_dw(idx)

    except Exception:
        logging.debug("error in add_image_overlay function")


def remove_image_overlay(self) -> None:
    """
    remove image overlay from all players
    """
    keys_to_delete: list = []
    for n_player in self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO].get(cfg.OVERLAY, {}):
        keys_to_delete.append(n_player)
        try:
            self.overlays[int(n_player) - 1].remove()
        except Exception:
            logging.debug("Error removing image overlay")
    for n_player in keys_to_delete:
        del self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO][cfg.OVERLAY][n_player]



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.

"""

from PySide6.QtGui import QColor
from PySide6.QtWidgets import (
    QTableWidgetItem,
    QLabel,
    QLineEdit,
    QTableWidget,
    QAbstractItemView,
    QComboBox,
    QGridLayout,
    QHBoxLayout,
    QSpacerItem,
    QPushButton,
    QDialog,
    QSizePolicy,
)

from . import config as cfg
from . import dialog

commands_index = {"Start": 2, "Edit": 3, "View": 4}


class MyTableWidgetItem(QTableWidgetItem):
    def __init__(self, text, sortKey):
        QTableWidgetItem.__init__(self, text, QTableWidgetItem.UserType)
        self.sortKey = sortKey

    # Qt uses a simple < check for sorting items, override this to use the sortKey
    def __lt__(self, other):
        if isinstance(self.sortKey, str) and isinstance(other.sortKey, str):
            return self.sortKey.lower() < other.sortKey.lower()
        else:
            return self.sortKey < other.sortKey


class observationsList_widget(QDialog):
    def __init__(
        self,
        data: list,
        header: list,
        column_type: list,
        not_paired: list = [],
        parent=None,
    ):
        super(observationsList_widget, self).__init__(parent)

        self.data = data
        self.not_paired = not_paired
        self.column_type = column_type

        self.setWindowTitle(f"Observations list - {cfg.programName}")
        self.label = QLabel("")

        self.mode = cfg.SINGLE

        self.lineEdit = QLineEdit(self)
        self.lineEdit.textChanged.connect(self.view_filter)
        self.view = QTableWidget(self)
        self.view.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.view.setSortingEnabled(True)

        self.comboBox = QComboBox(self)
        self.comboBox.currentIndexChanged.connect(self.view_filter)

        self.cbLogic = QComboBox(self)
        self.cbLogic.addItems(
            [
                "contains",
                "does not contain",
                "=",
                "!=",
                ">",
                "<",
                ">=",
                "<=",
                "between (use and to separate terms)",
            ]
        )
        self.cbLogic.currentIndexChanged.connect(self.view_filter)

        self.label = QLabel(self)

        self.gridLayout = QGridLayout(self)
        self.gridLayout.addWidget(self.label, 0, 0, 1, 3)
        self.gridLayout.addWidget(self.comboBox, 1, 0, 1, 1)
        self.gridLayout.addWidget(self.cbLogic, 1, 1, 1, 1)
        self.gridLayout.addWidget(self.lineEdit, 1, 2, 1, 1)

        self.gridLayout.addWidget(self.view, 2, 0, 1, 3)

        hbox2 = QHBoxLayout()

        spacerItem = QSpacerItem(241, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        hbox2.addItem(spacerItem)

        self.pbSelectAll = QPushButton("Select all")
        self.pbSelectAll.clicked.connect(lambda: self.pbSelection_clicked("select"))
        hbox2.addWidget(self.pbSelectAll)

        self.pbUnSelectAll = QPushButton("Unselect all")
        self.pbUnSelectAll.clicked.connect(lambda: self.pbSelection_clicked("unselect"))
        hbox2.addWidget(self.pbUnSelectAll)

        self.pbCancel = QPushButton(cfg.CANCEL, clicked=self.pbCancel_clicked)
        hbox2.addWidget(self.pbCancel)

        self.pbOpen = QPushButton("Start", clicked=self.pbOpen_clicked)
        hbox2.addWidget(self.pbOpen)

        self.pbView = QPushButton("View", clicked=self.pbView_clicked)
        hbox2.addWidget(self.pbView)

        self.pbEdit = QPushButton("Edit", clicked=self.pbEdit_clicked)
        hbox2.addWidget(self.pbEdit)

        self.pbOk = QPushButton(cfg.OK, clicked=self.pbOk_clicked)
        hbox2.addWidget(self.pbOk)

        self.gridLayout.addLayout(hbox2, 3, 0, 1, 3)

        self.view.doubleClicked.connect(self.view_doubleClicked)

        self.view.setRowCount(len(self.data))
        if self.data:
            self.view.setColumnCount(len(self.data[0]))

        self.view.setHorizontalHeaderLabels(header)

        for r in range(len(self.data)):
            for c in range(len(self.data[0])):
                self.view.setItem(r, c, self.set_item(r, c))

        self.view.resizeColumnsToContents()

        self.comboBox.addItems(header)

        self.view.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.label.setText(f"{self.view.rowCount()} observation{'s' * (self.view.rowCount() > 1)}")

    def view_doubleClicked(self, index):
        if self.mode == cfg.MULTIPLE:
            return

        if self.mode == cfg.OPEN or self.mode == cfg.EDIT:
            self.done(2)
            return

        if self.mode == cfg.SELECT1:
            self.done(2)
            return

        response = dialog.MessageDialog(
            cfg.programName,
            "What do you want to do with this observation?",
            list(commands_index.keys()) + [cfg.CANCEL],
        )
        if response == cfg.CANCEL:
            return
        else:
            self.done(commands_index[response])

    def pbSelection_clicked(self, mode):
        """
        select or unselect all filtered observations
        """
        if mode == "select":
            self.view.selectAll()
        if mode == "unselect":
            self.view.clearSelection()

    def pbCancel_clicked(self):
        self.close()

    def pbOk_clicked(self):
        self.done(1)

    def pbOpen_clicked(self):
        self.done(2)

    def pbEdit_clicked(self):
        self.done(3)

    def pbView_clicked(self):
        self.done(4)

    def set_item(self, r, c):
        if self.column_type[c] == cfg.NUMERIC:
            try:
                item = MyTableWidgetItem(self.data[r][c], float(self.data[r][c]))
            except Exception:
                item = MyTableWidgetItem(self.data[r][c], 0)
        else:
            item = MyTableWidgetItem(self.data[r][c], self.data[r][c])

        # if obs_id in not_paired -> set background color to red
        if c == 0 and self.data[r][c] in self.not_paired:
            item.setBackground(QColor(255, 0, 0, 128))
            # item.setForeground(QBrush(QColor(0, 255, 0)))

        return item

    def view_filter(self):
        """
        filter
        """

        def str2float(s: str):
            """
            convert str in float or return str
            """
            try:
                return float(s)
            except Exception:
                return s

        def in_(s, lst):
            return s in lst

        def not_in(s, lst):
            return s not in lst

        def equal(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num == s_num
            else:
                return x == s

        def not_equal(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num != s_num
            else:
                return x != s

        def gt(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num > s_num
            else:
                return x > s

        def lt(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num < s_num
            else:
                return x < s

        def gt_or_equal(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num >= s_num
            else:
                return x >= s

        def lt_or_equal(s, x):
            x_num, s_num = str2float(x), str2float(s)
            if type(x_num) == type(s_num):
                return x_num <= s_num
            else:
                return x <= s

        def between(s, x):
            if len(s.split(" AND ")) != 2:
                return None
            s1, s2 = s.split(" AND ")
            s1_num, s2_num = str2float(s1), str2float(s2)
            if type(s1_num) != type(s2_num):
                return None
            x_num = str2float(x)
            if type(s1_num) == type(x_num):
                return s1_num <= x_num <= s2_num
            else:
                return s1 <= x <= s2

        if not self.lineEdit.text():
            self.view.setRowCount(len(self.data))

            for r in range(len(self.data)):
                for c in range(len(self.data[0])):
                    self.view.setItem(r, c, self.set_item(r, c))

        else:
            if self.cbLogic.currentText() == "contains":
                logic = in_
            if self.cbLogic.currentText() == "does not contain":
                logic = not_in
            if self.cbLogic.currentText() == "=":
                logic = equal
            if self.cbLogic.currentText() == "!=":
                logic = not_equal
            if self.cbLogic.currentText() == ">":
                logic = gt
            if self.cbLogic.currentText() == "<":
                logic = lt
            if self.cbLogic.currentText() == ">=":
                logic = gt_or_equal
            if self.cbLogic.currentText() == "<=":
                logic = lt_or_equal
            if "between" in self.cbLogic.currentText():
                logic = between

            self.view.setRowCount(0)
            search = self.lineEdit.text().upper()
            try:
                for r, row in enumerate(self.data):
                    if logic(search, row[self.comboBox.currentIndex()].upper()):
                        self.view.setRowCount(self.view.rowCount() + 1)
                        for c, _ in enumerate(row):
                            self.view.setItem(self.view.rowCount() - 1, c, self.set_item(r, c))
            except Exception:
                pass
        self.label.setText(f"{self.view.rowCount()} observation{'s' * (self.view.rowCount() > 1)}")



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2025 Olivier Friard


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.

"""

import bisect
import logging
from decimal import Decimal as dec
import re
import pathlib as pl

from . import config as cfg
from . import dialog
from . import utilities as util
from . import select_modifiers
from . import event_operations


def write_event(self, event: dict, mem_time: dec) -> int:
    """
    add event from pressed key to observation
    offset is added to event time
    ask for modifiers if configured
    load events in tableview
    scroll to active event

    Args:
        event (dict): event parameters
        memTime (Decimal): time

    """

    logging.debug(f"write event - event: {event}  memtime: {mem_time}")

    if event is None:
        return 1

    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.LIVE:
        # live observation finished (end of time interval reached)
        if not self.liveObservationStarted and mem_time.is_nan():
            _ = dialog.MessageDialog(
                cfg.programName,
                (
                    "The live observation is finished.<br>"
                    "The observation interval is "
                    f"{self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])[0]} - "
                    f"{self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])[1]}"
                ),
                (cfg.OK,),
            )
            return 1

        print(f"{mem_time=}")

        if mem_time < self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])[0]:
            _ = dialog.MessageDialog(
                cfg.programName,
                (
                    "The live observation has not began.<br>"
                    "The observation interval is "
                    f"{self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])[0]} - "
                    f"{self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.OBSERVATION_TIME_INTERVAL, [0, 0])[1]}"
                ),
                (cfg.OK,),
            )
            return 1

    editing_event = "row" in event

    # add time offset if not from editing
    if not editing_event:
        # add offset
        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA, cfg.LIVE):
            mem_time += dec(self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TIME_OFFSET]).quantize(dec(".001"))

        # add media creation time
        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA):
            if self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.MEDIA_CREATION_DATE_AS_OFFSET, False):
                media_file_name = self.dw_player[0].player.playlist[self.dw_player[0].player.playlist_pos]["filename"]

                logging.debug(f"{media_file_name=}")

                media_file_name_posix = pl.Path(media_file_name).as_posix()

                logging.debug(f"{media_file_name_posix=}")

                # add media creation date/time

                """
                print(f"{media_file_name=}")
                print(f"{mem_time=}")
                """
                print(f"{self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO]=}")

                mem_time += dec(
                    self.pj[cfg.OBSERVATIONS][self.observationId][cfg.MEDIA_INFO][cfg.MEDIA_CREATION_TIME][media_file_name_posix]
                )

    # check if time > 2**31 - 1 (2147483647)
    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA, cfg.VIEWER_MEDIA, cfg.LIVE, cfg.VIEWER_LIVE):
        if (mem_time < -2147483647) or (mem_time > 2147483647):
            _ = dialog.MessageDialog(
                cfg.programName,
                (f"The timestamp must be between -2147483647 and 2147483647.<br>The current timestamp is {mem_time}"),
                (cfg.OK,),
            )
            return 1

    # remove key code from modifiers
    subject = event.get(cfg.SUBJECT, self.currentSubject)
    comment = event.get(cfg.COMMENT, "")

    if self.playerType in (cfg.IMAGES, cfg.VIEWER_IMAGES):
        image_idx = event.get(cfg.IMAGE_INDEX, "")
        image_path = event.get(cfg.IMAGE_PATH, "")
        # check if pictures dir is relative
        if str(pl.Path(image_path).parent) not in self.pj[cfg.OBSERVATIONS][self.observationId].get(cfg.DIRECTORIES_LIST, []):
            image_path = str(pl.Path(image_path).relative_to(pl.Path(self.projectFileName).parent))

    if self.playerType in (cfg.MEDIA, cfg.VIEWER_MEDIA):
        frame_idx = event.get(cfg.FRAME_INDEX, cfg.NA)

    # check if a same event is already in events list (time, subject, code)

    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA, cfg.LIVE):
        # adding event
        if (not editing_event) and self.checkSameEvent(
            self.observationId,
            mem_time,
            subject,
            event[cfg.BEHAVIOR_CODE],
        ):
            _ = dialog.MessageDialog(cfg.programName, "The same event already exists (same time, behavior code and subject).", (cfg.OK,))
            return 1

        # modifying event and time was changed
        if editing_event and mem_time != self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][event["row"]][0]:
            if self.checkSameEvent(
                self.observationId,
                mem_time,
                subject,
                event[cfg.BEHAVIOR_CODE],
            ):
                _ = dialog.MessageDialog(
                    cfg.programName,
                    "The same event already exists (same time, behavior code and subject).",
                    [cfg.OK],
                )
                return 1

    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.IMAGES:
        # adding event
        if (not editing_event) and self.checkSameEvent(
            self.observationId,
            image_idx,
            subject,
            event[cfg.BEHAVIOR_CODE],
        ):
            _ = dialog.MessageDialog(
                cfg.programName,
                "The same event already exists (same image index, behavior code and subject).",
                [cfg.OK],
            )
            return 1

        # modifying event and time was changed
        if (
            editing_event
            and image_idx
            != self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][event["row"]][cfg.PJ_OBS_FIELDS[cfg.IMAGES][cfg.IMAGE_INDEX]]
        ):
            if self.checkSameEvent(
                self.observationId,
                image_idx,
                subject,
                event[cfg.BEHAVIOR_CODE],
            ):
                _ = dialog.MessageDialog(
                    cfg.programName,
                    "The same event already exists (same image index, behavior code and subject).",
                    (cfg.OK,),
                )
                return 1

    # extract State events
    state_behaviors_codes = util.state_behavior_codes(self.pj[cfg.ETHOGRAM])

    # index of current subject
    # subject_idx = self.subject_name_index[self.currentSubject] if self.currentSubject else ""

    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.LIVE, cfg.MEDIA):
        position = mem_time
    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.IMAGES:
        position = dec(image_idx)  # decimal to pass to util.get_current_states_modifiers_by_subject

    current_states: dict = util.get_current_states_modifiers_by_subject(
        state_behaviors_codes,
        self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
        dict(self.pj[cfg.SUBJECTS], **{"": {"name": ""}}),
        position,
        include_modifiers=False,
    )

    # check if ask modifiers at stop is enabled
    flag_ask_at_stop = False
    if event["type"] == cfg.STATE_EVENT:
        for idx in event[cfg.MODIFIERS]:
            if event[cfg.MODIFIERS][idx].get("ask at stop", False):
                flag_ask_at_stop = True
                break

    flag_ask_modifier = False
    if flag_ask_at_stop:
        # TODO: check if new event is a STOP one

        idx_subject: str = ""
        for idx in current_states:
            if idx in self.pj[cfg.SUBJECTS] and self.pj[cfg.SUBJECTS][idx][cfg.SUBJECT_NAME] == self.currentSubject:
                idx_subject = idx
                break

        if event[cfg.BEHAVIOR_CODE] in current_states[idx_subject]:
            flag_ask_modifier = True
    else:
        flag_ask_modifier = True

    if flag_ask_modifier:
        if "from map" in event:  # modifiers only for behaviors without coding map
            modifier_str = event["from map"]
        else:
            # check if event has modifiers
            modifier_str: str = ""

            if event[cfg.MODIFIERS]:
                selected_modifiers: dict = {}
                modifiers_external_data: dict = {}
                # check if modifiers are from external data
                for idx in event[cfg.MODIFIERS]:
                    if event[cfg.MODIFIERS][idx]["type"] == cfg.EXTERNAL_DATA_MODIFIER:
                        if editing_event:
                            original_modifiers_list = event.get("original_modifiers", "").split("|")
                            modifiers_external_data[idx] = dict(event[cfg.MODIFIERS][idx])
                            modifiers_external_data[idx]["selected"] = original_modifiers_list[int(idx)]

                        else:  # no edit
                            for idx2 in self.plot_data:
                                if self.plot_data[idx2].y_label.upper() == event[cfg.MODIFIERS][idx]["name"].upper():
                                    modifiers_external_data[idx] = dict(event[cfg.MODIFIERS][idx])
                                    modifiers_external_data[idx]["selected"] = self.plot_data[idx2].lb_value.text()

                # check if modifiers are in single, multiple or numeric
                if [x for x in event[cfg.MODIFIERS] if event[cfg.MODIFIERS][x]["type"] != cfg.EXTERNAL_DATA_MODIFIER]:
                    # pause media
                    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA):
                        if self.playerType == cfg.MEDIA:
                            if self.dw_player[0].player.pause:
                                memState = cfg.PAUSED
                            elif self.dw_player[0].player.time_pos is not None:
                                memState = cfg.PLAYING
                            else:
                                memState = cfg.STOPPED
                            if memState == cfg.PLAYING:
                                self.pause_video()

                    # check if editing (original_modifiers key)
                    currentModifiers = event.get("original_modifiers", "")

                    modifiers_selector = select_modifiers.ModifiersList(
                        event[cfg.BEHAVIOR_CODE], eval(str(event[cfg.MODIFIERS])), currentModifiers
                    )

                    r = modifiers_selector.exec_()
                    if r:
                        selected_modifiers = modifiers_selector.get_modifiers()

                    # restart media
                    if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.MEDIA:
                        if self.playerType == cfg.MEDIA:
                            if memState == cfg.PLAYING:
                                self.play_video()
                    if not r:  # cancel button pressed
                        return

                all_modifiers = {**selected_modifiers, **modifiers_external_data}

                modifier_str: str = ""
                for idx in util.sorted_keys(all_modifiers):
                    if modifier_str:
                        modifier_str += "|"
                    if all_modifiers[idx]["type"] in (cfg.SINGLE_SELECTION, cfg.MULTI_SELECTION):
                        modifier_str += ",".join(all_modifiers[idx].get("selected", ""))
                    if all_modifiers[idx]["type"] in (cfg.NUMERIC_MODIFIER, cfg.EXTERNAL_DATA_MODIFIER):
                        modifier_str += all_modifiers[idx].get("selected", "NA")

        modifier_str = re.sub(r" \(.*\)", "", modifier_str)
    else:  # do not ask modifier
        modifier_str = ""

    # update current state
    # TODO: verify event["subject"] / self.currentSubject

    # print(f"{current_states=}")

    # logging.debug(f"self.currentSubject {self.currentSubject}")
    # logging.debug(f"current_states {current_states}")

    # fill the undo list
    event_operations.fill_events_undo_list(self, "Undo last event edition" if editing_event else "Undo last event insertion")

    logging.debug("save list of events for undo operation")

    if not editing_event:
        if self.currentSubject:
            csj: list = []  # list of current state for the current subject
            for idx in current_states:
                if idx in self.pj[cfg.SUBJECTS] and self.pj[cfg.SUBJECTS][idx][cfg.SUBJECT_NAME] == self.currentSubject:
                    csj = current_states[idx]
                    break

        else:  # no focal subject
            try:
                csj = current_states[""]
            except Exception:
                csj = []

        logging.debug(f"csj {csj}")

        # print(f"{csj=}")

        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.LIVE, cfg.MEDIA):
            check_index = cfg.PJ_OBS_FIELDS[self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE]][cfg.TIME]
        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.IMAGES:
            check_index = cfg.PJ_OBS_FIELDS[cfg.IMAGES][cfg.IMAGE_INDEX]

        cm: dict = {}  # modifiers for current behaviors
        for cs in csj:
            for ev in self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS]:
                if ev[check_index] > position:
                    break

                if ev[cfg.EVENT_SUBJECT_FIELD_IDX] == self.currentSubject:
                    if ev[cfg.EVENT_BEHAVIOR_FIELD_IDX] == cs:
                        cm[cs] = ev[cfg.EVENT_MODIFIER_FIELD_IDX]

        if flag_ask_at_stop:
            # set modifier to START behavior
            if event[cfg.BEHAVIOR_CODE] in csj:
                mem_idx = -1
                for idx, e in enumerate(self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS]):
                    # print(f"{e=}")
                    if e[0] >= mem_time:
                        break
                    # same behavior, same subject and modifier(s) not set
                    if e[2] == event[cfg.BEHAVIOR_CODE] and e[1] == self.currentSubject and e[3] == "":
                        mem_idx = idx
                if mem_idx != -1:
                    self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][mem_idx][3] = modifier_str
                    csj.remove(event[cfg.BEHAVIOR_CODE])

        for cs in csj:
            # close state if same state without modifier
            if (
                self.close_the_same_current_event
                and (event[cfg.BEHAVIOR_CODE] == cs)
                and modifier_str.replace("None", "").replace("|", "") == ""
            ):
                modifier_str = cm[cs]
                continue

            if (event[cfg.EXCLUDED] and cs in event[cfg.EXCLUDED].split(",")) or (
                event[cfg.BEHAVIOR_CODE] == cs and cm[cs] != modifier_str
            ):
                # check if behavior to stop is a 'ask modifier at stop'
                behavior_to_stop = [
                    self.pj[cfg.ETHOGRAM][x]
                    for x in self.pj[cfg.ETHOGRAM]
                    if self.pj[cfg.ETHOGRAM][x][cfg.BEHAVIOR_CODE] == cs
                    and self.pj[cfg.ETHOGRAM][x]["type"] in (cfg.STATE_EVENT, cfg.STATE_EVENT_WITH_CODING_MAP)
                ]
                if behavior_to_stop:
                    behavior_to_stop = behavior_to_stop[0]

                    flag_behavior_ask_at_stop = False
                    for idx in behavior_to_stop[cfg.MODIFIERS]:
                        if behavior_to_stop[cfg.MODIFIERS][idx].get("ask at stop", False):
                            flag_behavior_ask_at_stop = True
                            break
                    if flag_behavior_ask_at_stop:
                        modifiers_selector = select_modifiers.ModifiersList(
                            behavior_to_stop[cfg.BEHAVIOR_CODE],
                            eval(str(behavior_to_stop[cfg.MODIFIERS])),
                            currentModifier="",
                        )

                        r = modifiers_selector.exec_()
                        if r:
                            selected_modifiers = modifiers_selector.get_modifiers()
                            # print(f"{selected_modifiers=}")

                            behavior_to_stop_modifier_str: str = ""
                            for idx in util.sorted_keys(selected_modifiers):
                                if behavior_to_stop_modifier_str:
                                    behavior_to_stop_modifier_str += "|"
                                if selected_modifiers[idx]["type"] in (cfg.SINGLE_SELECTION, cfg.MULTI_SELECTION):
                                    behavior_to_stop_modifier_str += ",".join(selected_modifiers[idx].get("selected", ""))
                                if selected_modifiers[idx]["type"] in (cfg.NUMERIC_MODIFIER, cfg.EXTERNAL_DATA_MODIFIER):
                                    behavior_to_stop_modifier_str += selected_modifiers[idx].get("selected", "NA")

                            # set the start modifier
                            mem_idx = -1
                            for idx, e in enumerate(self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS]):
                                if e[0] >= mem_time - dec("0.001"):
                                    break
                                # same behavior, same subject and modifier(s) not set
                                if e[2] == behavior_to_stop[cfg.BEHAVIOR_CODE] and e[1] == self.currentSubject and e[3] == "":
                                    mem_idx = idx
                            if mem_idx != -1:
                                self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][mem_idx][3] = behavior_to_stop_modifier_str

                    else:
                        behavior_to_stop_modifier_str = cm[cs]

                # add excluded state event to observations (= STOP them)
                if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.LIVE):
                    bisect.insort(
                        self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
                        [mem_time - dec("0.001"), self.currentSubject, cs, behavior_to_stop_modifier_str, ""],
                    )

                if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.MEDIA):
                    bisect.insort(
                        self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
                        [mem_time - dec("0.001"), self.currentSubject, cs, behavior_to_stop_modifier_str, "", cfg.NA],
                    )

                if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] in (cfg.IMAGES):
                    self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].append(
                        [mem_time, self.currentSubject, cs, behavior_to_stop_modifier_str, "", image_idx, image_path]
                    )

                    # order by image index ASC
                    self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].sort(
                        key=lambda x: x[cfg.PJ_OBS_FIELDS[self.playerType][cfg.IMAGE_INDEX]]
                    )

    # add event to pj
    if editing_event:  # modifying event
        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.MEDIA:
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][event["row"]] = [
                mem_time,
                subject,
                event[cfg.BEHAVIOR_CODE],
                modifier_str,
                comment,
                frame_idx,
            ]
            # order by image index ASC
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].sort()

        elif self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.LIVE:
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][event["row"]] = [
                mem_time,
                subject,
                event[cfg.BEHAVIOR_CODE],
                modifier_str,
                comment,
            ]
            # order by image index ASC
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].sort()

        elif self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.IMAGES:
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS][event["row"]] = [
                mem_time,
                subject,
                event[cfg.BEHAVIOR_CODE],
                modifier_str,
                comment,
                image_idx,
                image_path,
            ]
            # order by image index ASC
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].sort(
                key=lambda x: x[cfg.PJ_OBS_FIELDS[self.playerType][cfg.IMAGE_INDEX]]
            )

    else:  # add event
        if self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.MEDIA:
            bisect.insort(
                self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
                [mem_time, subject, event[cfg.BEHAVIOR_CODE], modifier_str, comment, frame_idx],
            )
        elif self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.LIVE:
            bisect.insort(
                self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
                [mem_time, subject, event[cfg.BEHAVIOR_CODE], modifier_str, comment],
            )

        elif self.pj[cfg.OBSERVATIONS][self.observationId][cfg.TYPE] == cfg.IMAGES:
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].append(
                [mem_time, subject, event[cfg.BEHAVIOR_CODE], modifier_str, comment, image_idx, image_path]
            )
            # order by image index ASC
            self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS].sort(
                key=lambda x: x[cfg.PJ_OBS_FIELDS[self.playerType][cfg.IMAGE_INDEX]]
            )

    # reload all events in tw
    self.load_tw_events(self.observationId)

    self.project_changed()

    self.get_events_current_row()

    # index of current subject selected by observer
    subject_idx = self.subject_name_index[self.currentSubject] if self.currentSubject else ""

    self.currentStates = util.get_current_states_modifiers_by_subject(
        self.state_behaviors_codes,
        self.pj[cfg.OBSERVATIONS][self.observationId][cfg.EVENTS],
        dict(self.pj[cfg.SUBJECTS], **{"": {"name": ""}}),
        self.getLaps(),
        include_modifiers=True,
    )

    self.lbCurrentStates.setText(f"Observed behaviors: {', '.join(self.currentStates[subject_idx])}")
    # show current states in subjects table
    self.show_current_states_in_subjects_table()

    return 0
